Package: EQUALrepeat
Type: Package
Title: Algorithm Driven Time Series Analysis for Researchers without
        Coding Skills
Version: 0.4.5
Date: 2026-05-18
Authors@R: person("Kurinchi", "Gurusamy", role = c("aut", "cre"), email = "k.gurusamy@ucl.ac.uk")
Author: Kurinchi Gurusamy [aut, cre]
Maintainer: Kurinchi Gurusamy <k.gurusamy@ucl.ac.uk>
Depends: stringr, stats, ggplot2, DescTools
Imports: zip, cowplot, rstatix, forecast, tseries, urca, vars,
        viridisLite
Description: Support functions for R-based 'EQUAL-STATS' software which automatically classifies the data and performs appropriate statistical tests. 'EQUAL-STATS' software is a shiny application with an user-friendly interface to perform complex statistical analysis. Gurusamy,K (2024)<doi:10.5281/zenodo.13354162>.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://sites.google.com/view/equal-group/home
Note: This update removes dependencies on irr package.
NeedsCompilation: no
Packaged: 2026-05-18 08:03:39 UTC; kurin
Repository: CRAN
Date/Publication: 2026-05-18 08:50:02 UTC
Built: R 4.6.0; ; 2026-05-18 10:06:48 UTC; unix

Package: EQUALrepeat
Type: Package
Title: Algorithm Driven Time Series Analysis for Researchers without
        Coding Skills
Version: 0.4.5
Date: 2026-05-18
Authors@R: person("Kurinchi", "Gurusamy", role = c("aut", "cre"), email = "k.gurusamy@ucl.ac.uk")
Author: Kurinchi Gurusamy [aut, cre]
Maintainer: Kurinchi Gurusamy <k.gurusamy@ucl.ac.uk>
Depends: stringr, stats, ggplot2, DescTools
Imports: zip, cowplot, rstatix, forecast, tseries, urca, vars,
        viridisLite
Description: Support functions for R-based 'EQUAL-STATS' software which automatically classifies the data and performs appropriate statistical tests. 'EQUAL-STATS' software is a shiny application with an user-friendly interface to perform complex statistical analysis. Gurusamy,K (2024)<doi:10.5281/zenodo.13354162>.
License: GPL (>= 3)
Encoding: UTF-8
URL: https://sites.google.com/view/equal-group/home
Note: This update removes dependencies on irr package.
NeedsCompilation: no
Packaged: 2026-05-18 08:03:39 UTC; kurin
Repository: CRAN
Date/Publication: 2026-05-18 08:50:02 UTC
Built: R 4.6.0; ; 2026-05-18 09:32:11 UTC; unix

export(function.rbind_different_column_numbers)
export(round_near)
export(function.read_data)
export(function.Measurement_Error)
export(function.Repeated_Measures)
export(function.Time_Series)
import(stringr)
import(stats)
import(ggplot2)
import(DescTools)
importFrom(utils, read.csv, write.table)
importFrom(cowplot, plot_grid, ggdraw, draw_label, as_grob)
importFrom(zip, zip)
importFrom(rstatix, get_anova_table)
importFrom(forecast, forecast, auto.arima, findfrequency, ndiffs, nsdiffs)
importFrom(tseries, adf.test)
importFrom(urca, ca.jo)
importFrom(vars, vec2var, VAR)
importFrom(viridisLite, viridis)

function.rbind_different_column_numbers <- function(list, include_columns) {
  x = list
  if (missing(include_columns)) {
    include_columns <- "Yes"
  }
  if (length(x) < 2) {
    output <- x[[1]]
  } else {
    # Default for combining multiple data frames has to have the same column numbers and the column names have to be the same
    # This can be problematic when multiple data frames have to added to the same data frame row-wise
    # This function overcomes this problem by changing the column names to default, creating a header from the name of the list (if available) and adding column names from each data frame as a new row
    if (is.null(names(x))) {
      headers <- rep("", length(x))
    } else {
      headers <- names(x)
    }
    # get the maximum columns across different data frames
    column_numbers <- sapply(1:length(x),function(z) {ncol(x[[z]])})
    max_columns <- max(column_numbers)
    z <- list()
    output_each_data_frame <- list()
    # Process the files in such a way that is easy to convert to html format later
    for (i in 1:length(x)) {
      z[[3]] <- data.frame(x[[i]], check.names = FALSE)
      if (column_numbers[i] < max_columns) {
        z[[3]][,(column_numbers[i]+1):max_columns] <- NA
        colnames(z[[3]])[(column_numbers[i]+1):max_columns] <- ""
      }
      if (headers[i] != "") {
        z[[1]] < -data.frame(matrix(NA, nrow = 1, ncol = max_columns), row.names = paste0("Header_",i))
        z[[1]][1,1] <- headers[i]
        colnames(z[[1]]) <- paste0("V",1:max_columns)
      }
      if (include_columns == "Yes") {
        z[[2]] <- data.frame(matrix(NA, nrow = 1, ncol = max_columns), row.names = paste0("Column_names_",i))
        z[[2]][1,] <- colnames(z[[3]])
        colnames(z[[2]]) <- paste0("V",1:max_columns)
      }
      colnames(z[[3]]) <- paste0("V",1:max_columns)
      output_each_data_frame[[i]] <- do.call(rbind.data.frame,z)
    }
    output <- do.call(rbind.data.frame, output_each_data_frame)
  }
  return(output)
}

globalVariables(c("Time", "Average", "outcome", "subject_id", "time_of_measurement", "group"), "EQUALrepeat", add = TRUE)
function.Repeated_Measures <- function(Predefined_lists, rv){
  # Lists
  plan <- {cbind.data.frame(
    analysis_number = paste0("AN", formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0)),
    first_menu_choice = rv$first_menu_choice,
    second_menu_choice = rv$second_menu_choice,
    entry_1 = paste0(rv$entry[[1]], collapse = "%_%"),
    entry_2 = paste0(rv$entry[[2]], collapse = "%_%"),
    entry_3 = paste0(rv$entry[[3]], collapse = "%_%"),
    entry_4 = paste0(rv$entry[[4]], collapse = "%_%"),
    entry_5 = paste0(rv$entry[[5]], collapse = "%_%"),
    entry_6 = paste0(rv$entry[[6]], collapse = "%_%"),
    entry_7 = paste0(rv$entry[[7]], collapse = "%_%"),
    entry_8 = paste0(rv$entry[[8]], collapse = "%_%"),
    entry_9 = paste0(rv$entry[[9]], collapse = "%_%"),
    entry_10 = paste0(rv$entry[[10]], collapse = "%_%"),
    entry_11 = paste0(rv$entry[[11]], collapse = "%_%"),
    entry_12 = paste0(rv$entry[[12]], collapse = "%_%"),
    entry_13 = paste0(rv$entry[[13]], collapse = "%_%"),
    entry_14 = paste0(rv$entry[[14]], collapse = "%_%"),
    entry_15 = paste0(rv$entry[[15]], collapse = "%_%"),
    same_row_different_row = rv$same_row_different_row
  )}
  selections <- {paste0(
    '<b>entry_1: </b>', paste0(rv$entry[[1]], collapse = "; "), '<br>',
    '<b>entry_2: </b>', paste0(rv$entry[[2]], collapse = "; "), '<br>',
    '<b>entry_3: </b>', paste0(rv$entry[[3]], collapse = "; "), '<br>',
    '<b>entry_4: </b>', paste0(rv$entry[[4]], collapse = "; "), '<br>',
    '<b>entry_5: </b>', paste0(rv$entry[[5]], collapse = "; "), '<br>'
  )}
  code <- {paste0(
    '# AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0), '\n',
    'rv$first_menu_choice <- "', rv$first_menu_choice, '"\n',
    'rv$second_menu_choice <- ', ifelse(is.na(rv$second_menu_choice),NA,paste0('"',rv$second_menu_choice, '"')), '\n',
    'rv$entry[[1]] <- ', ifelse(length(rv$entry[[1]]) > 1,
                                paste0('c("', paste0(rv$entry[[1]], collapse = '", "'), '")'),
                                paste0('"',rv$entry[[1]],'"')), '\n',
    'rv$entry[[2]] <- ', ifelse(length(rv$entry[[2]]) > 1,
                                paste0('c("', paste0(rv$entry[[2]], collapse = '", "'), '")'),
                                paste0('"',rv$entry[[2]],'"')), '\n',
    'rv$entry[[3]] <- ', ifelse(length(rv$entry[[3]]) > 1,
                                paste0('c("', paste0(rv$entry[[3]], collapse = '", "'), '")'),
                                paste0('"',rv$entry[[3]],'"')), '\n',
    'rv$entry[[4]] <- ', ifelse(length(rv$entry[[4]]) > 1,
                                paste0('c("', paste0(rv$entry[[4]], collapse = '", "'), '")'),
                                paste0('"',rv$entry[[4]],'"')), '\n',
    'rv$entry[[5]] <- ', ifelse(length(rv$entry[[5]]) > 1,
                                paste0('c("', paste0(rv$entry[[5]], collapse = '", "'), '")'),
                                paste0('"',rv$entry[[5]],'"')), '\n',
    'rv$same_row_different_row <- "', rv$same_row_different_row, '"\n',
    'AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0), '_results <- function.',rv$first_menu_choice,'(Predefined_lists, rv)', '\n',
    if(length(rv$plan) == 0){
      'if (TRUE %in% (AN0001_results$plots_list != "")) {invisible(file.rename(AN0001_results$plots_list, paste0(AN0001_results$plots_list,"_copy")))}
'
    } else {
      paste0(
        'AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$results[2,1] <- "AN',formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'"', '\n',
        'if (TRUE %in% (AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$plots_list != "")) {invisible(file.rename(AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$plots_list, str_replace_all(AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$plots_list, "/AN0001_", "/AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_")))}', '\n')
    },
    'write.table(x = AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$results, append = TRUE, file = paste0(rv$StorageFolder, "/results.csv"), sep = ",", row.names = FALSE, col.names = FALSE, na = "", quote = FALSE, qmethod = "double")', '\n'
  )}
  # Check normality
  func.check.normality <- function(variable) {
    # Test normality through skewness, kurtosis, Shapiro-Wilk test, and Kolmogrov-Smirnov test
    # Skewness - the confidence intervals do not overlap (-0.5) to 0.5
    skewness <- suppressWarnings(try(Skew(variable, conf.level = 0.95), silent = TRUE))
    if (str_detect(skewness[[1]][1], "Error")) {
      skewness <- "Not possible to determine"
    } else if ((skewness[3] < (-0.5)) | (skewness[2] > 0.5)) {
      skewness <- "Non-normal"
    } else {
      skewness <- "No evidence that it is non-normal"
    }
    kurtosis <- suppressWarnings(try(Kurt(variable, conf.level = 0.95), silent = TRUE))
    if (str_detect(kurtosis[[1]][1], "Error")) {
      kurtosis <- "Not possible to determine"
    } else if ((kurtosis[2] + 3) > 3) { # kurtosis provided is excess kurtosis. To get the actual kurtosis add 3
      kurtosis <- "Non-normal"
    } else {
      kurtosis <- "No evidence that it is non-normal"
    }
    # Shapiro-Wilk test: This may not work for more than 5000 observations
    shapiro.wilk <- suppressWarnings(try(shapiro.test(variable[!is.na(variable)]), silent = TRUE))
    if (str_detect(shapiro.wilk[[1]][1], "Error")) {
      if (length(variable) > 5000) {
        shapiro.wilk <- "No evidence that it is non-normal"
      } else {
        shapiro.wilk <- "Not possible to determine"
      }
    } else if (shapiro.wilk$p.value <= 0.10) {
      shapiro.wilk <- "Non-normal"
    } else {
      shapiro.wilk <- "No evidence that it is non-normal"
    }
    ks <- suppressWarnings(ks.test(variable,"pnorm", mean=mean(variable), sd=sd(variable)))
    if (is.na(ks$p.value)) {
      ks <- "Not possible to determine"
    } else if (ks$p.value <= 0.10) {
      ks <- "Non-normal"
    } else {
      ks <- "No evidence that it is non-normal"
    }
    normality_results <- c(skewness, kurtosis, shapiro.wilk, ks)
    table(normality_results)
    # Now at least one is "Non-normal" or at least three have "Not possible to determine"
    if (
      ("Non-normal" %in% normality_results) |
      (length(normality_results[normality_results == "Not possible to determine"]) > 2)
    ) {
      distribution <- "Non-normal"
    } else {
      distribution <- "No evidence that it is non-normal"
    }
    return(distribution)
  }
  # Some generic processing of data to remove unrepresented factors
  func.keep_present_categories_only <- function(variable_values, is.ordinal) {
    factors_in_variable <- levels(variable_values)
    factors_in_variable <- factors_in_variable[! is.na(match(factors_in_variable, variable_values))]
    factor(as.character(variable_values), levels = factors_in_variable, ordered = is.ordinal)
  }
  # Summary
  func.summary.categorical <- function(variable, prefix) {
    if(nlevels(variable) == 2) {
      summary <- data.frame(sapply(
        1:nlevels(variable), function(x) {
          as.numeric(BinomCI(length(variable[(!is.na(variable)) & variable == levels(variable)[x]]), length(variable[! is.na(variable)]), conf.level = 0.95))
        }
      ), check.names = FALSE)
      colnames(summary) <- paste0(prefix, ": ", levels(variable))
      row.names(summary) <- c("Proportion", "Proportion - LCI", "Proportion - UCI")
    } else {
      summary <- data.frame(t(MultinomCI(table(variable), conf.level = 0.95)),row.names = c("Proportion", "Proportion - LCI", "Proportion - UCI"), check.names = FALSE)
      colnames(summary) <- paste0(prefix, ": ", colnames(summary))
    }
    return(summary)
  }
  func.summary.quantitative <- function(variable, prefix, normality) {
    if (length(variable) == 1) {
      summary <- c(prefix, variable, NA, NA)
    } else {
      if (normality == TRUE) {
        mean_ci <- suppressWarnings(try(MeanCI(variable, conf.level = 0.95), silent = TRUE))
        if (length(mean_ci) < 3) {mean_ci <- c(NA, NA, NA)}
        summary <- c(prefix, as.numeric(mean_ci))
      } else {
        median_ci <- suppressWarnings(try(MedianCI(variable, conf.level = 0.95), silent = TRUE))
        if (length(median_ci) < 3) {median_ci <- c(NA, NA, NA)}
        summary <- c(prefix, as.numeric(median_ci))
      }
    }
    return(summary)
  }
  # As there are multiple situations for unsuccessful outcome, create a function for unsuccessful outcome
  func.unsuccessful.outcome <- function(reason) {
    results_display <- data.frame(
      `Analysis number` = paste0("AN",formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0)),
      `Analysis type` = paste0(Predefined_lists$main_menu[Predefined_lists$menu_short ==  rv$first_menu_choice], ifelse(! is.na(rv$second_menu_choice), paste0(": ", rv$second_menu_choice), "")),
      `Analysis outcome` = "Unsuccessful",
      `Reason for unsuccesful analysis` = reason,
      check.names = FALSE
    )
    results <- rbind.data.frame(
      colnames(results_display),
      results_display
    )
    plots_list <- ""
    plots_list_display <- plots_list
    analysis_outcome <- "Unsuccessful"
    display_plot <- FALSE
    display_table <- TRUE
    function_output <- list(analysis_outcome = analysis_outcome, plan = plan, code = code, results = results, results_display = results_display, plots_list = plots_list,   plots_list_display =   plots_list_display, selections = selections, display_table = display_table, display_plot = display_plot)
  }
  func.successful.outcome <- function(data, test_results) {
    results <- data.frame(
      `Analysis number` = paste0("AN",formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0)),
      `Analysis type` = paste0(Predefined_lists$main_menu[Predefined_lists$menu_short ==  rv$first_menu_choice], ifelse(! is.na(rv$second_menu_choice), paste0(": ", rv$second_menu_choice), "")),
      `Analysis outcome` = "Successful",
      check.names = FALSE
    )
    descriptive_summary_display <- descriptive_summary
    differences_display <- differences
    test_results_display <- test_results
    results_display <- results
    descriptive_summary_display <- data.frame(lapply(descriptive_summary_display[,1:ncol(descriptive_summary_display)], function(x){round(as.numeric(x),4)}))
    colnames(descriptive_summary_display) <- colnames(descriptive_summary)
    differences_display[,2:4] <- sapply(differences_display[,2:4], function(x){round(as.numeric(x),4)})
    if (rv$entry[[4]] != "") {
      test_results_display[,3:4] <- sapply(test_results_display[,3:4], function(x){round(as.numeric(x),4)})
    } else {
      test_results_display[,2:3] <- sapply(test_results_display[,2:3], function(x){round(as.numeric(x),4)})
    }
    results <- function.rbind_different_column_numbers(list(results, descriptive_summary, test_results, differences))
    results_display <- function.rbind_different_column_numbers(list(results_display, descriptive_summary_display, test_results_display, differences_display))
    if (rv$same_row_different_row == "Same row") {plot_title <- paste0(rv$entry[[2]], " versus ", rv$entry[[3]])} else {plot_title <- paste0(rv$entry[[2]], " over time")}
    if (rv$entry[[2]] %in% rv$import_data$categorical) {
      data_2 <- data.frame(table(data$outcome, data$time_of_measurement))
      if (rv$same_row_different_row == "Same row") {colnames(data_2)[1:2] <- c("Variable value", "Time")} else {colnames(data_2)[1:2] <- c(rv$entry[[2]],rv$entry[[3]])}
      plot <- ggplot(data_2, aes(x= data_2[,2], y= data_2[,3], fill= data_2[,1], group= data_2[,1]))+ geom_bar(width = 0.7, stat = "identity", position = position_dodge())+ theme(axis.title.x=element_blank(), axis.title.y=element_blank(),axis.ticks.x=element_blank(), plot.title = element_text(color="navyblue", size=14, face="bold", hjust = 0.5)) + ggtitle(plot_title) + labs(fill=rv$entry[[1]]) + geom_text(aes(label = paste0(round(data_2[,3]/nrow(data),2)*100,"%")), size=3, position = position_dodge(width = 0.7), vjust = -0.5) + labs(fill=rv$entry[[1]]) + scale_fill_discrete()
    } else {
      if (rv$entry[[4]] != "") {
        data_2 <- data.frame(Average = as.numeric(t(descriptive_summary[,sequence(nlevels(data$time_of_measurement)*nlevels(data$group), 1, 3)])),
                             Time = unlist(lapply(1:nlevels(data$time_of_measurement), function(x) {rep(levels(data$time_of_measurement)[x],nlevels(data$group))})),
                             Group = rep(levels(data$group), nlevels(data$time_of_measurement))
        )
      } else {
        data_2 <- data.frame(Average = as.numeric(t(descriptive_summary[,sequence(nlevels(data$time_of_measurement), 1, 3)])), Time = levels(data$time_of_measurement))
      }
      lower_limit <- ifelse(min(data_2$Average) < 0, pretty(min(data_2$Average)), 0)
      upper_limit <- pretty(data_2$Average)[length(pretty(data_2$Average))]
      y_ticks <- pretty(lower_limit:upper_limit, n = 4)
      lower_limit <- min(y_ticks)
      upper_limit <- max(y_ticks)
      if (rv$entry[[4]] != "") {
        plot_title <- paste0(plot_title, "\n(by ", rv$entry[[4]], ")")
        each_plot <- list()
        data_3 <- list()
        for (i in 1:nlevels(data$group)) {
          data_3[[i]] <- data_2[data_2$Group == levels(data$group)[i],]
          each_plot[[i]] <- ggplot(data_3[[i]], aes(x = Time, y = Average, fill= Time)) + geom_bar(position = "dodge", stat="identity", width = 0.7, show.legend = FALSE) +
            theme(plot.title = element_text(color="navyblue", size=8, face="bold", hjust = 0.5)) +
            ylab("Mean") + labs(fill=levels(data_3[[i]]$Time)) + ggtitle(levels(data$group)[i]) + theme(axis.title.x = element_blank()) + scale_y_continuous(limits = c(lower_limit, upper_limit), breaks = y_ticks)
        }
        plot_title_combined <- ggdraw() + draw_label(plot_title, color="navyblue", size=14, fontface="bold", hjust = 0.5) + theme(plot.margin = margin(0, 0, 0, 7))
        plot <- plot_grid(plot_title_combined, plot_grid(plotlist = each_plot, ncol = 2) + theme(plot.background = element_rect(fill = "white", colour = NA)), ncol = 1, rel_heights = c(0.1,1)) + theme(plot.background = element_rect(fill = "white", colour = NA))
      } else {
        plot <- ggplot(data_2, aes(x = Time, y = Average, fill = Time)) + geom_bar(position = "dodge", stat="identity") +
          theme(plot.title = element_text(color="navyblue", size=14, face="bold", hjust = 0.5)) +
          ylab(ifelse(((repeated_measures_test == "Paired-sample T-test") | (repeated_measures_test == "Repeated Measures ANOVA")), "Mean", "Median")) +
          labs(fill=levels(data_2$Categories)) + ggtitle(plot_title) + theme(axis.title.x = element_blank()) + scale_y_continuous(limits = c(lower_limit, upper_limit), breaks = y_ticks)
      }
    }
    suppressWarnings(suppressMessages(ggsave(filename = paste0(rv$StorageFolder,'/AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0), '_',
                                                               substr(str_replace_all(plot_title, "[^[:alnum:]]","_"), 1, 80) ,'.png'),
                                             plot = plot)))
    plots_list <- paste0(rv$StorageFolder,'/AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0), '_',
                         substr(str_replace_all(plot_title, "[^[:alnum:]]","_"), 1, 80) ,'.png')
    plots_list_display <- plots_list
    analysis_outcome <- "Successful"
    display_plot <- TRUE
    display_table <- TRUE
    function_output <- list(analysis_outcome = analysis_outcome, plan = plan, code = code, results = results, results_display = results_display, plots_list = plots_list,   plots_list_display =   plots_list_display, selections = selections, display_table = display_table, display_plot = display_plot)
  }
  func.ci_differences_paired_data_categorical <- function(data, alpha) {
    z = qnorm(1-alpha/2)
    # Convert data to wide format
    subject_id <- unique(data$subject_id)
    data_wide_list <- lapply(1:nlevels(data$time_of_measurement), function(y) {
      outcome <- data[data$time_of_measurement == levels(data$time_of_measurement)[y],1:2]
      colnames(outcome) <- c("subject_id", levels(data$time_of_measurement)[y])
      return(outcome)
    })
    data_wide <- data_wide_list[[1]]
    for (i in 2:nlevels(data$time_of_measurement)) {data_wide <- merge(data_wide, data_wide_list[[i]])}
    data_wide <- data_wide[,2:ncol(data_wide)]
    contingency_tables_list_1 <- lapply(2:nlevels(data$time_of_measurement), function(y) {
      data_contingency_table <- cbind.data.frame(data_wide[,1], data_wide[,y])
      contingency_table <- table(data_contingency_table)
    })
    names(contingency_tables_list_1) <- paste0(levels(data$time_of_measurement)[2:nlevels(data$time_of_measurement)], " vs ", levels(data$time_of_measurement)[1])
    # If the data is ordinal, cumulative proportions versus reference else each category versus reference
    if (rv$entry[[3]] %in% rv$import_data$ordinal) {
      contingency_tables_list_final <- lapply(1:length(contingency_tables_list_1), function(z) {
        contingency_table_nxn <- contingency_tables_list_1[[z]]
        contingency_table_2x2 <- lapply(2:nrow(contingency_table_nxn), function(x) {
          c(paste0(names(contingency_tables_list_1)[z], ": ",
                   if (x == 2) {
                     colnames(contingency_table_nxn)[2]
                   } else if (x == 3) {
                     paste0(colnames(contingency_table_nxn)[2:x], collapse = " or ")
                   } else {
                     paste0(paste0(colnames(contingency_table_nxn)[2:(x-1)], collapse = ", "), ", or ",  colnames(contingency_table_nxn)[x])
                   },
                   " vs ", colnames(contingency_table_nxn)[1]),
            sum(contingency_table_nxn[2:x,2:x]),
            sum(contingency_table_nxn[1,2:x]),
            sum(contingency_table_nxn[2:x,1]),
            contingency_table_nxn[1,1]
          )
        })
        if (nrow(contingency_table_nxn) > 2) {
          contingency_table_2x2 <- do.call(rbind.data.frame,contingency_table_2x2)
        } else {
          contingency_table_2x2 <- data.frame(matrix(data = contingency_table_2x2[[1]], nrow = 1, ncol = 5))
        }
        colnames(contingency_table_2x2) <- c("Comparison", "r", "s", "t", "u")
        return(contingency_table_2x2)
      })
    } else {
      contingency_tables_list_final <- lapply(1:length(contingency_tables_list_1), function(z) {
        contingency_table_nxn <- contingency_tables_list_1[[z]]
        contingency_table_2x2 <- lapply(2:nrow(contingency_table_nxn), function(x) {
          c(paste0(names(contingency_tables_list_1)[z], ": ", colnames(contingency_table_nxn)[x], " vs ", colnames(contingency_table_nxn)[1]),
            contingency_table_nxn[x,x],
            contingency_table_nxn[1,x],
            sum(contingency_table_nxn[x,1]),
            contingency_table_nxn[1,1]
          )
        })
        if (nrow(contingency_table_nxn) > 2) {
          contingency_table_2x2 <- do.call(rbind.data.frame,contingency_table_2x2)
        } else {
          contingency_table_2x2 <- data.frame(matrix(data = contingency_table_2x2[[1]], nrow = 1, ncol = 5))
        }
        colnames(contingency_table_2x2) <- c("Comparison", "r", "s", "t", "u")
        return(contingency_table_2x2)
      })
    }
    if (length(contingency_tables_list_final) == 1) {contingency_tables <- contingency_tables_list_final[[1]]} else {contingency_tables <- do.call(rbind.data.frame,contingency_tables_list_final)}
    contingency_tables[,c("r", "s", "t", "u")] <- lapply(contingency_tables[,c("r", "s", "t", "u")], as.numeric)
    contingency_tables$n <- rowSums(contingency_tables[,c("r", "s", "t", "u")])
    contingency_tables$pt <- (contingency_tables$r + contingency_tables$s)/contingency_tables$n
    contingency_tables$qt <- 1-contingency_tables$pt
    contingency_tables$At <- 2 * (contingency_tables$r + contingency_tables$s) + z^2
    contingency_tables$Bt <- z*(z^2 + 4 * (contingency_tables$r + contingency_tables$s) * contingency_tables$qt)^0.5
    contingency_tables$Ct <- 2 *(contingency_tables$n + z^2)
    contingency_tables$lt <- (contingency_tables$At - contingency_tables$Bt)/contingency_tables$Ct
    contingency_tables$ut <- (contingency_tables$At + contingency_tables$Bt)/contingency_tables$Ct
    contingency_tables$pc <- (contingency_tables$r + contingency_tables$t)/contingency_tables$n
    contingency_tables$qc <- 1-contingency_tables$pc
    contingency_tables$Ac <- 2 * (contingency_tables$r + contingency_tables$t) + z^2
    contingency_tables$Bc <- z*(z^2 + 4 * (contingency_tables$r + contingency_tables$t) * contingency_tables$qc)^0.5
    contingency_tables$Cc <- 2 *(contingency_tables$n + z^2)
    contingency_tables$lc <- (contingency_tables$Ac - contingency_tables$Bc)/contingency_tables$Cc
    contingency_tables$uc <- (contingency_tables$Ac + contingency_tables$Bc)/contingency_tables$Cc
    contingency_tables$D <- (contingency_tables$s - contingency_tables$t)/contingency_tables$n
    contingency_tables$phi <- sapply(1:nrow(contingency_tables), function(x){
      if (((contingency_tables$r[x] + contingency_tables$s[x]) == 0) | ((contingency_tables$t[x] + contingency_tables$u[x]) == 0) | ((contingency_tables$r[x] + contingency_tables$t[x]) == 0) | ((contingency_tables$s[x] + contingency_tables$u[x]) == 0)){
        0
      } else {
        A <- (contingency_tables$r[x] + contingency_tables$s[x]) * (contingency_tables$t[x] + contingency_tables$u[x]) * (contingency_tables$r[x] + contingency_tables$t[x]) * (contingency_tables$s[x] + contingency_tables$u[x])
        B <- (contingency_tables$r[x] * contingency_tables$u[x]) - (contingency_tables$s[x] * contingency_tables$t[x])
        if (B > contingency_tables$n[x]/2) {
          C <- B - contingency_tables$n[x]/2
        } else if (B < 0) {
          C <- B
        } else {
          C <- 0
        }
        C/A^0.5
      }
    })
    contingency_tables$LCI <- contingency_tables$D - ((contingency_tables$pt - contingency_tables$lt)^2 - 2 * contingency_tables$phi * (contingency_tables$pt - contingency_tables$lt) * (contingency_tables$uc - contingency_tables$pc) + (contingency_tables$uc - contingency_tables$pc)^2)^0.5
    contingency_tables$UCI <- contingency_tables$D + ((contingency_tables$pc - contingency_tables$lc)^2 - 2 * contingency_tables$phi * (contingency_tables$pc - contingency_tables$lc) * (contingency_tables$ut - contingency_tables$pt) + (contingency_tables$ut - contingency_tables$pt)^2)^0.5
    differences <- cbind.data.frame(
      Comparison = contingency_tables$Comparison,
      `Differences in proportion` = contingency_tables$D,
      `Differences in proportion - LCI` = contingency_tables$LCI,
      `Differences in proportion - UCI` = contingency_tables$UCI
    )
    return(differences)
  }
  func.ci_differences_paired_data_quantitative <- function(data, alpha, repeated_measures_test) {
    if ((repeated_measures_test == "Paired-sample T-test") | (repeated_measures_test == "Repeated Measures ANOVA")) {normality <- TRUE} else {normality <- FALSE}
    if (normality == TRUE) {
      differences <- lapply(2:nlevels(data$time_of_measurement), function(x) {
        test_results <- t.test(data$outcome[data$time_of_measurement == levels(data$time_of_measurement)[1]], data$outcome[data$time_of_measurement == levels(data$time_of_measurement)[x]], paired = TRUE, alpha = alpha)
        c(paste0(if(rv$same_row_different_row == "Same row") {paste0(rv$entry[[3]], " vs ", rv$entry[[2]])} else {rv$entry[[2]]}, ": ",
                 levels(data$time_of_measurement)[x], " vs ", levels(data$time_of_measurement)[1]
        ),
        -test_results$estimate,
        -test_results$conf.int[2],
        -test_results$conf.int[1]
        )
      })
      if (length(differences) > 1) {differences <- do.call(rbind.data.frame, differences)} else {differences <- data.frame(matrix(data = differences[[1]], nrow = 1, ncol = 4))   }
      colnames(differences) <- c("Difference in means: Comparison", "Mean difference", "Mean difference - LCI", "Mean difference - UCI")
    } else {
      # Convert data to wide format
      subject_id <- unique(data$subject_id)
      data_wide_list <- lapply(1:nlevels(data$time_of_measurement), function(y) {
        outcome <- data[data$time_of_measurement == levels(data$time_of_measurement)[y],1:2]
        colnames(outcome) <- c("subject_id", levels(data$time_of_measurement)[y])
        return(outcome)
      })
      data_wide <- data_wide_list[[1]]
      for (i in 2:nlevels(data$time_of_measurement)) {data_wide <- merge(data_wide, data_wide_list[[i]])}
      data_wide <- data_wide[,2:ncol(data_wide)]
      differences <- lapply(2:nlevels(data$time_of_measurement), function(y) {
        differences_individual <- data_wide[,y] - data_wide[,1]
        median_ci <- suppressWarnings(try(MedianCI(differences_individual, conf.level = (1- alpha), method = "exact", na.rm = TRUE), silent = TRUE))
        if (str_detect(median_ci[[1]][1], "Error")) {
          c(paste0(if(rv$same_row_different_row == "Same row") {paste0(rv$entry[[3]], " vs ", rv$entry[[2]])} else {rv$entry[[2]]}, ": ",
                   levels(data$time_of_measurement)[y], " vs ", levels(data$time_of_measurement)[1]
          ),
          rep("Not estimable", 3)
          )
        } else {
          c(paste0(if(rv$same_row_different_row == "Same row") {paste0(rv$entry[[3]], " vs ", rv$entry[[2]])} else {rv$entry[[2]]}, ": ",
                   levels(data$time_of_measurement)[y], " vs ", levels(data$time_of_measurement)[1]
          ),
          as.numeric(median_ci)
          )
        }
      })
      if (length(differences) > 1) {differences <- do.call(rbind.data.frame, differences)} else {differences <- data.frame(matrix(data = differences[[1]], nrow = 1, ncol = 4))   }
      colnames(differences) <- c("Difference in medians: Comparison", "Median difference", "Median difference - LCI", "Median difference - UCI")
    }
    return(differences)
  }
  if (rv$entry[[5]] == "") {alpha = 0.05} else {alpha = as.numeric(rv$entry[[5]])}
  # Now with the data
  data_initial <- rv$import_data$data[,c(rv$entry[[1]], rv$entry[[2]], rv$entry[[3]], if(rv$entry[[4]]!=""){rv$entry[[4]]})]
  data_initial <- na.omit(data_initial)
  # Convert all data to long format
  if (rv$same_row_different_row == "Same row") {
    data = cbind.data.frame(
      subject_id = as.factor(c(data_initial[,1], data_initial[,1])),
      outcome = c(data_initial[,2], data_initial[,3]),
      time_of_measurement = as.factor(c(rep(rv$entry[[2]], nrow(data_initial)), rep(rv$entry[[3]], nrow(data_initial))))
    )
    if(rv$entry[[4]]!="") {data$group <- c(data_initial[,4], data_initial[,4])}
  } else {
    data <- data.frame(data_initial)
    if (! is.factor(data[,1])) {factor(data[,1])}
    colnames(data) <- c("subject_id", "outcome", "time_of_measurement", if(rv$entry[[4]]!="") {"group"})
  }
  # Do the analysis only if at least 2 subjects
  if (nlevels(data$subject_id) >= 2) {
    # Keep only the present categories
    if (rv$entry[[2]] %in% rv$import_data$categorical) {data$outcome <- func.keep_present_categories_only(variable_values = data$outcome, is.ordinal = (rv$entry[[2]] %in% rv$import_data$ordinal))}
    data$time_of_measurement <- func.keep_present_categories_only(variable_values = data$time_of_measurement, is.ordinal = TRUE)
    if(rv$entry[[4]]!="") {data$group <- func.keep_present_categories_only(variable_values = data$group, is.ordinal = (rv$entry[[2]] %in% rv$import_data$ordinal))}
    # If EQUAL-STATS choice, find which test to perform
    if (rv$second_menu_choice == "EQUAL-STATS choice") {
      if (rv$entry[[2]] %in% rv$import_data$categorical) {
        if ((nlevels(data$outcome) == 2) & (nlevels(data$time_of_measurement) == 2)) {
          repeated_measures_test <- "McNemar test"
        } else {
          repeated_measures_test <- "Cochran Q-test"
        }
      } else {
        distribution <- func.check.normality(data$outcome)
        if (nlevels(data$time_of_measurement) == 2) {
          # If normal, paired T-test else Wilcoxon signed-rank test
          if (distribution == "No evidence that it is non-normal"){
            repeated_measures_test <- "Paired-sample T-test"
          } else {
            repeated_measures_test <- "Wilcoxon signed-rank test"
          }
        } else {
          # If normal, ANOVA else Friedman test
          if (distribution == "No evidence that it is non-normal"){
            repeated_measures_test <- "Repeated Measures ANOVA"
          } else {
            repeated_measures_test <- "Friedman test"
          }
        }
      }
    } else {
      repeated_measures_test <- rv$second_menu_choice
    }
    if (repeated_measures_test != "Repeated Measures ANOVA") {
      if (rv$entry[[2]] %in% rv$import_data$categorical) {
        descriptive_summary <- lapply(1:nlevels(data$time_of_measurement), function(x) {
          summary <- func.summary.categorical(variable = data$outcome[data$time_of_measurement == levels(data$time_of_measurement)[x]], prefix = levels(data$time_of_measurement)[x])
          summary_all_rows <- data.frame(t(summary), check.names = FALSE)
          summary_each_row <- lapply(1:nrow(summary_all_rows), function(z) {
            each_row <- cbind.data.frame(summary_all_rows[z,])
            colnames(each_row) <- paste0(row.names(each_row), " (", colnames(each_row), ")")
            return(each_row)
          })
          summary <- do.call(cbind.data.frame, summary_each_row)
        })
        descriptive_summary <- do.call(cbind.data.frame, descriptive_summary)
        differences <- func.ci_differences_paired_data_categorical(data, alpha)
      } else {
        descriptive_summary <- lapply(1:nlevels(data$time_of_measurement), function(x) {
          summary <- func.summary.quantitative(variable = data$outcome[data$time_of_measurement == levels(data$time_of_measurement)[x]],
                                               prefix = levels(data$time_of_measurement)[x],
                                               normality = (repeated_measures_test == "Paired-sample T-test")
          )
          summary_data_frame <- cbind.data.frame(summary[2], summary[3], summary[4])
          if (repeated_measures_test == "Paired-sample T-test") {
            colnames(summary_data_frame) <- paste0(summary[1], ": ", c("Mean", "Mean - LCI", "Mean - UCI"))
          } else {
            colnames(summary_data_frame) <- paste0(summary[1], ": ", c("Median", "Median - LCI", "Median - UCI"))
          }
          return(summary_data_frame)
        })
        descriptive_summary <- do.call(cbind.data.frame, descriptive_summary)
        differences <- func.ci_differences_paired_data_quantitative(data, alpha, repeated_measures_test)
      }
    } else {
      if (rv$entry[[4]] != "") {
        descriptive_summary <- lapply(1:nlevels(data$time_of_measurement), function(x) {
          summary_each_group <- lapply(1:nlevels(data$group), function(z) {
            summary <- func.summary.quantitative(variable = data$outcome[(data$time_of_measurement == levels(data$time_of_measurement)[x]) & (data$group == levels(data$group)[z])],
                                                 prefix = paste0("Group - ", levels(data$group)[z], " & ", "Time - ", levels(data$time_of_measurement)[x]),
                                                 normality = TRUE
            )
            summary_data_frame <- cbind.data.frame(summary[2], summary[3], summary[4])
            colnames(summary_data_frame) <- paste0(summary[1], ": ", c("Mean", "Mean - LCI", "Mean - UCI"))
            return(summary_data_frame)
          })
          summary_each_group <- do.call(cbind.data.frame, summary_each_group)
          return(summary_each_group)
        })
        descriptive_summary <- do.call(cbind.data.frame, descriptive_summary)
        differences <- lapply(1:nlevels(data$group), function(z) {
          each_group <- func.ci_differences_paired_data_quantitative(data[data$group == levels(data$group)[z],], alpha, repeated_measures_test)
          each_group[,1] <- paste0(levels(data$group)[z], ": ", paste0(levels(data$time_of_measurement)[2:nlevels(data$time_of_measurement)]), " vs ", levels(data$time_of_measurement)[1])
          return(each_group)
        })
        differences <- do.call(rbind.data.frame, differences)
      } else {
        descriptive_summary <- lapply(1:nlevels(data$time_of_measurement), function(x) {
          summary <- func.summary.quantitative(variable = data$outcome[data$time_of_measurement == levels(data$time_of_measurement)[x]],
                                               prefix = levels(data$time_of_measurement)[x],
                                               normality = TRUE
          )
          summary_data_frame <- cbind.data.frame(summary[2], summary[3], summary[4])
          colnames(summary_data_frame) <- paste0(summary[1], ": ", c("Mean", "Mean - LCI", "Mean - UCI"))
          return(summary_data_frame)
        })
        descriptive_summary <- do.call(cbind.data.frame, descriptive_summary)
        differences <- func.ci_differences_paired_data_quantitative(data, alpha, repeated_measures_test)
      }
    }
    if (repeated_measures_test == "McNemar test") {
      if (nlevels(data$time_of_measurement) == 2) {
        test_results <- mcnemar.test(data$outcome[data$time_of_measurement == levels(data$time_of_measurement)[1]], data$outcome[data$time_of_measurement == levels(data$time_of_measurement)[2]], correct = TRUE)
        test_results <- cbind.data.frame(
          Test = repeated_measures_test,
          `Test statistic` = test_results$statistic,
          `P value` = test_results$p.value
        )
        function_output <- func.successful.outcome(data, test_results)
      } else {
        function_output <- func.unsuccessful.outcome("The number of timepoints should be exactly two. Please try Cochran Q-test if the number of time points is more than two. There must be at least two valid observations at each timepoint.")
      }
    } else if (repeated_measures_test == "Cochran Q-test") {
      test_results <- suppressMessages(suppressWarnings(try(CochranQTest(outcome ~ time_of_measurement | subject_id, data=data), silent = TRUE)))
      if (suppressWarnings(str_detect(test_results[[1]][1],"Error"))) {
        function_output <- func.unsuccessful.outcome("There was an error in running the test. This may be due to the values of the data.")
      } else {
        test_results <- cbind.data.frame(
          Test = repeated_measures_test,
          `Test statistic` = test_results$statistic,
          `P value` = test_results$p.value
        )
        function_output <- func.successful.outcome(data, test_results)
      }
    } else if (repeated_measures_test == "Paired-sample T-test") {
      if (nlevels(data$time_of_measurement) == 2) {
        test_results <- t.test(data$outcome[data$time_of_measurement == levels(data$time_of_measurement)[1]], data$outcome[data$time_of_measurement == levels(data$time_of_measurement)[2]], paired = TRUE)
        test_results <- cbind.data.frame(
          Test = repeated_measures_test,
          `Test statistic` = test_results$statistic,
          `P value` = test_results$p.value
        )
        function_output <- func.successful.outcome(data, test_results)
      } else {
        function_output <- func.unsuccessful.outcome("The number of timepoints should be exactly two. Please try Repeated Measures ANOVA if the number of time points is more than two. There must be at least two valid observations at each timepoint.")
      }
    } else if (repeated_measures_test == "Repeated Measures ANOVA") {
      if (rv$entry[[4]] != "") {
        test_results <- suppressMessages(suppressWarnings(try(rstatix::get_anova_table(rstatix::anova_test(data = data, dv = outcome, wid = subject_id, within = time_of_measurement, between = group)), silent = TRUE)))
        if (suppressWarnings(str_detect(test_results[[1]][1],"Error"))) {
          function_output <- func.unsuccessful.outcome("There was an error in running the test. This may be due to the values of the data.")
        } else {
          test_results <- cbind.data.frame(
            Test = c(repeated_measures_test, rep(NA,2)),
            Variable = c(rv$entry[[4]], paste0("Change over time: ", rv$entry[[3]], " vs ", rv$entry[[2]]), paste0("Difference in change over time: ", rv$entry[[3]], " vs ", rv$entry[[2]], " between different categories of ", rv$entry[[4]])),
            `Test statistic` = test_results$F,
            `P value` = test_results$p
          )
          function_output <- func.successful.outcome(data, test_results)
        }
      } else {
        test_results <- suppressMessages(suppressWarnings(try(rstatix::get_anova_table(rstatix::anova_test(data = data, dv = outcome, wid = subject_id, within = time_of_measurement)), silent = TRUE)))
        if (suppressWarnings(str_detect(test_results[[1]][1],"Error"))) {
          function_output <- func.unsuccessful.outcome("There was an error in running the test. This may be due to the values of the data.")
        } else {
          test_results <- cbind.data.frame(
            Test = repeated_measures_test,
            Variable = paste0("Change over time: ", rv$entry[[3]], " vs ", rv$entry[[2]]),
            `Test statistic` = test_results$F,
            `P value` = test_results$p
          )
          function_output <- func.successful.outcome(data, test_results)
        }
      }
    } else if (repeated_measures_test == "Wilcoxon signed-rank test") {
      if (nlevels(data$time_of_measurement) == 2) {
        test_results <- suppressWarnings(wilcox.test(data$outcome[data$time_of_measurement == levels(data$time_of_measurement)[1]], data$outcome[data$time_of_measurement == levels(data$time_of_measurement)[2]], paired = TRUE))
        test_results <- cbind.data.frame(
          Test = repeated_measures_test,
          `Test statistic` = test_results$statistic,
          `P value` = test_results$p.value
        )
        function_output <- func.successful.outcome(data, test_results)
      } else {
        function_output <- func.unsuccessful.outcome("The number of timepoints should be exactly two. Please try Friedman test if the number of time points is more than two. There must be at least two valid observations at each timepoint.")
      }
    } else if (repeated_measures_test == "Friedman test") {
      test_results <- suppressMessages(suppressWarnings(try(friedman.test(outcome ~ time_of_measurement | subject_id, data=data), silent = TRUE)))
      if (suppressWarnings(str_detect(test_results[[1]][1],"Error"))) {
        function_output <- func.unsuccessful.outcome("There was an error in running the test. This may be due to the values of the data.")
      } else {
        test_results <- cbind.data.frame(
          Test = repeated_measures_test,
          `Test statistic` = test_results$statistic,
          `P value` = test_results$p.value
        )
        function_output <- func.successful.outcome(data, test_results)
      }
    }
  } else {
    function_output <- func.unsuccessful.outcome("Insufficient number of observations")
  }
  return(function_output)
}

globalVariables(c("ACF", "Lag"), "EQUALrepeat", add = TRUE)
function.Time_Series <- function(Predefined_lists, rv){
  # Lists
  plan <- {cbind.data.frame(
    analysis_number = paste0("AN", formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0)),
    first_menu_choice = rv$first_menu_choice,
    second_menu_choice = rv$second_menu_choice,
    entry_1 = paste0(rv$entry[[1]], collapse = "%_%"),
    entry_2 = paste0(rv$entry[[2]], collapse = "%_%"),
    entry_3 = paste0(rv$entry[[3]], collapse = "%_%"),
    entry_4 = paste0(rv$entry[[4]], collapse = "%_%"),
    entry_5 = paste0(rv$entry[[5]], collapse = "%_%"),
    entry_6 = paste0(rv$entry[[6]], collapse = "%_%"),
    entry_7 = paste0(rv$entry[[7]], collapse = "%_%"),
    entry_8 = paste0(rv$entry[[8]], collapse = "%_%"),
    entry_9 = paste0(rv$entry[[9]], collapse = "%_%"),
    entry_10 = paste0(rv$entry[[10]], collapse = "%_%"),
    entry_11 = paste0(rv$entry[[11]], collapse = "%_%"),
    entry_12 = paste0(rv$entry[[12]], collapse = "%_%"),
    entry_13 = paste0(rv$entry[[13]], collapse = "%_%"),
    entry_14 = paste0(rv$entry[[14]], collapse = "%_%"),
    entry_15 = paste0(rv$entry[[15]], collapse = "%_%"),
    same_row_different_row = ""
  )}
  selections <- {paste0(
    '<b>entry_1: </b>', paste0(rv$entry[[1]], collapse = "; "), '<br>',
    '<b>entry_2: </b>', paste0(rv$entry[[2]], collapse = "; "), '<br>',
    '<b>entry_3: </b>', paste0(rv$entry[[3]], collapse = "; "), '<br>',
    '<b>entry_4: </b>', paste0(rv$entry[[4]], collapse = "; "), '<br>',
    '<b>entry_5: </b>', paste0(rv$entry[[5]], collapse = "; "), '<br>',
    '<b>entry_6: </b>', paste0(rv$entry[[6]], collapse = "; "), '<br>'
  )}
  code <- {paste0(
    '# AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0), '\n',
    'rv$first_menu_choice <- "', rv$first_menu_choice, '"\n',
    'rv$second_menu_choice <- ', ifelse(is.na(rv$second_menu_choice),NA,paste0('"',rv$second_menu_choice, '"')), '\n',
    'rv$entry[[1]] <- ', ifelse(length(rv$entry[[1]]) > 1,
                                paste0('c("', paste0(rv$entry[[1]], collapse = '", "'), '")'),
                                paste0('"',rv$entry[[1]],'"')), '\n',
    'rv$entry[[2]] <- ', ifelse(length(rv$entry[[2]]) > 1,
                                paste0('c("', paste0(rv$entry[[2]], collapse = '", "'), '")'),
                                paste0('"',rv$entry[[2]],'"')), '\n',
    'rv$entry[[3]] <- ', ifelse(length(rv$entry[[3]]) > 1,
                                paste0('c("', paste0(rv$entry[[3]], collapse = '", "'), '")'),
                                paste0('"',rv$entry[[3]],'"')), '\n',
    'rv$entry[[4]] <- ', ifelse(length(rv$entry[[4]]) > 1,
                                paste0('c("', paste0(rv$entry[[4]], collapse = '", "'), '")'),
                                paste0('"',rv$entry[[4]],'"')), '\n',
    'rv$entry[[5]] <- ', ifelse(length(rv$entry[[5]]) > 1,
                                paste0('c("', paste0(rv$entry[[5]], collapse = '", "'), '")'),
                                paste0('"',rv$entry[[5]],'"')), '\n',
    'rv$entry[[6]] <- ', paste0(rv$entry[[6]], '\n'),
    'AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0), '_results <- function.',rv$first_menu_choice,'(Predefined_lists, rv)', '\n',
    if(length(rv$plan) == 0){
      'if (TRUE %in% (AN0001_results$plots_list != "")) {invisible(file.rename(AN0001_results$plots_list, paste0(AN0001_results$plots_list,"_copy")))}
'
    } else {
      paste0(
        'AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$results[2,1] <- "AN',formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'"', '\n',
        'if (TRUE %in% (AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$plots_list != "")) {invisible(file.rename(AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$plots_list, str_replace_all(AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$plots_list, "/AN0001_", "/AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_")))}', '\n')
    },
    'write.table(x = AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$results, append = TRUE, file = paste0(rv$StorageFolder, "/results.csv"), sep = ",", row.names = FALSE, col.names = FALSE, na = "", quote = FALSE, qmethod = "double")', '\n'
  )}
  func.recognise_date <- function(data) {
    lapply(1:ncol(data), function(y) {
      if (is.numeric(data[,y])) {
        field_type <- "numeric"
        converted <- NA
      } else {
        character <- as.character(data[,y])
        character <- character[! is.na(character)]
        if (length(character) > 0) {
          # Number of characters must be 10 in a date field
          characters_count <- nchar(character)
          if (length(unique(characters_count)) > 1) {
            field_type <- "non-date character field"
            converted <- NA
          } else if (unique(characters_count) != 10) {
            field_type <- "non-date character field"
            converted <- NA
          } else {
            # If forward slash is present, it must be exactly two
            forward_slash_count <- str_count(character, "/")
            if (length(unique(forward_slash_count)) > 1) {
              field_type <- "non-date character field"
              converted <- NA
            } else if (unique(forward_slash_count) != 2) {
              # If hyphen is present, it must be exactly two
              hyphen_count <- str_count(character, "-")
              if (length(unique(hyphen_count)) > 1) {
                field_type <- "non-date character field"
                converted <- NA
              } else if (unique(hyphen_count) != 2) {
                # If dot is present, it must be exactly two
                dot_count <- str_count(character, "\\.")
                if (length(unique(dot_count)) > 1) {
                  field_type <- "non-date character field"
                  converted <- NA
                } else if (unique(dot_count) != 2) {
                  field_type <- "non-date character field"
                  converted <- NA
                } else {
                  field_split <- do.call(rbind.data.frame, str_split(character,"\\."))
                  colnames(field_split) <- paste0("X",1:3)
                  # Number of characters
                  nchar1 <- nchar(field_split$X1)
                  nchar2 <- nchar(field_split$X2)
                  nchar3 <- nchar(field_split$X3)
                  unique_nchar1 <- unique(nchar1)
                  unique_nchar2 <- unique(nchar2)
                  unique_nchar3 <- unique(nchar3)
                  if ((length(unique_nchar1) > 1) | (length(unique_nchar2) > 1) | (length(unique_nchar3) > 1)) {
                    field_type <- "non-date character field"
                    converted <- NA
                  } else {
                    combination <- paste0(unique_nchar1, unique_nchar2, unique_nchar3)
                    if (combination == "224") {
                      # Check whether the first field or second field is month
                      converted <- try(as.POSIXct(paste0(field_split$X3, "-", field_split$X2, "-", field_split$X1), tz = "GMT"), silent = TRUE)
                      if (str_detect(converted[[1]][1], "Error")) {
                        converted <- try(as.POSIXct(paste0(field_split$X3, "-", field_split$X1, "-", field_split$X2), tz = "GMT"), silent = TRUE)
                      }
                      if (str_detect(converted[[1]][1], "Error")) {
                        field_type <- "non-date character field"
                        converted <- NA
                      } else {
                        field_type <- "date"
                      }
                    } else if (combination == "422") {
                      converted <- try(as.POSIXct(paste0(field_split$X1, "-", field_split$X2, "-", field_split$X3), tz = "GMT"), silent = TRUE)
                      if (str_detect(converted[[1]][1], "Error")) {
                        field_type <- "non-date character field"
                        converted <- NA
                      } else {
                        field_type <- "date"
                      }
                    } else {
                      field_type <- "non-date character field"
                      converted <- NA
                    }
                  }
                }
              } else {
                field_split <- do.call(rbind.data.frame, str_split(character,"-"))
                colnames(field_split) <- paste0("X",1:3)
                # Number of characters
                nchar1 <- nchar(field_split$X1)
                nchar2 <- nchar(field_split$X2)
                nchar3 <- nchar(field_split$X3)
                unique_nchar1 <- unique(nchar1)
                unique_nchar2 <- unique(nchar2)
                unique_nchar3 <- unique(nchar3)
                if ((length(unique_nchar1) > 1) | (length(unique_nchar2) > 1) | (length(unique_nchar3) > 1)) {
                  field_type <- "non-date character field"
                  converted <- NA
                } else {
                  combination <- paste0(unique_nchar1, unique_nchar2, unique_nchar3)
                  if (combination == "224") {
                    # Check whether the first field or second field is month
                    converted <- try(as.POSIXct(paste0(field_split$X3, "-", field_split$X2, "-", field_split$X1), tz = "GMT"), silent = TRUE)
                    if (str_detect(converted[[1]][1], "Error")) {
                      converted <- try(as.POSIXct(paste0(field_split$X3, "-", field_split$X1, "-", field_split$X2), tz = "GMT"), silent = TRUE)
                    }
                    if (str_detect(converted[[1]][1], "Error")) {
                      field_type <- "non-date character field"
                      converted <- NA
                    } else {
                      field_type <- "date"
                    }
                  } else if (combination == "422") {
                    converted <- try(as.POSIXct(paste0(field_split$X1, "-", field_split$X2, "-", field_split$X3), tz = "GMT"), silent = TRUE)
                    if (str_detect(converted[[1]][1], "Error")) {
                      field_type <- "non-date character field"
                      converted <- NA
                    } else {
                      field_type <- "date"
                    }
                  } else {
                    field_type <- "non-date character field"
                    converted <- NA
                  }
                }
              }
            } else {
              field_split <- do.call(rbind.data.frame, str_split(character,"/"))
              colnames(field_split) <- paste0("X",1:3)
              # Number of characters
              nchar1 <- nchar(field_split$X1)
              nchar2 <- nchar(field_split$X2)
              nchar3 <- nchar(field_split$X3)
              unique_nchar1 <- unique(nchar1)
              unique_nchar2 <- unique(nchar2)
              unique_nchar3 <- unique(nchar3)
              if ((length(unique_nchar1) > 1) | (length(unique_nchar2) > 1) | (length(unique_nchar3) > 1)) {
                field_type <- "non-date character field"
                converted <- NA
              } else {
                combination <- paste0(unique_nchar1, unique_nchar2, unique_nchar3)
                if (combination == "224") {
                  # Check whether the first field or second field is month
                  converted <- try(as.POSIXct(paste0(field_split$X3, "-", field_split$X2, "-", field_split$X1), tz = "GMT"), silent = TRUE)
                  if (str_detect(converted[[1]][1], "Error")) {
                    converted <- try(as.POSIXct(paste0(field_split$X3, "-", field_split$X1, "-", field_split$X2), tz = "GMT"), silent = TRUE)
                  }
                  if (str_detect(converted[[1]][1], "Error")) {
                    field_type <- "non-date character field"
                    converted <- NA
                  } else {
                    field_type <- "date"
                  }
                } else if (combination == "422") {
                  converted <- try(as.POSIXct(paste0(field_split$X1, "-", field_split$X2, "-", field_split$X3), tz = "GMT"), silent = TRUE)
                  if (str_detect(converted[[1]][1], "Error")) {
                    field_type <- "non-date character field"
                    converted <- NA
                  } else {
                    field_type <- "date"
                  }
                } else {
                  field_type <- "non-date character field"
                  converted <- NA
                }
              }
            }
          }
        } else {
          field_type <- "Unknown"
          converted <- NA
        }
      }
      output <- list(field_type = field_type, converted = converted)
      return(output)
    })
  }
  func.recognise_time <- function(data) {
    sapply(1:ncol(data), function(y) {
      if (is.numeric(data[,y])) {
        time <- FALSE
      } else {
        character <- as.character(data[,y])
        character <- character[! is.na(character)]
        if (length(character) > 0) {
          characters_count <- nchar(character)
          if (length(unique(characters_count)) > 1) {
            time <- FALSE
          } else if (unique(characters_count) != 8) {
            time <- FALSE
          } else {
            colon_count <- str_count(character, ":")
            if (length(unique(colon_count)) > 1) {
              time <- FALSE
            } else if (unique(colon_count) != 2) {
              time <- FALSE
            } else {
              field_split <- do.call(rbind.data.frame, str_split(character,":"))
              colnames(field_split) <- paste0("X",1:3)
              # Number of characters
              nchar1 <- nchar(field_split$X1)
              nchar2 <- nchar(field_split$X2)
              nchar3 <- nchar(field_split$X3)
              unique_nchar1 <- unique(nchar1)
              unique_nchar2 <- unique(nchar2)
              unique_nchar3 <- unique(nchar3)
              if ((length(unique_nchar1) > 1) | (length(unique_nchar2) > 1) | (length(unique_nchar3) > 1)) {
                time <- FALSE
              } else if ((unique_nchar1 != 2) | (unique_nchar2 != 2) | (unique_nchar1 != 2)) {
                time <- FALSE
              } else if ((TRUE %in% (field_split$X1 > 23)) | (TRUE %in% (field_split$X2 > 59)) | (TRUE %in% (field_split$X1 > 59))) {
                time <- FALSE
              } else {
                time <- TRUE
              }
            }
          }
        }
      }
    })
  }
  func.convert_to_date <- function(number) {
    x <- as.POSIXlt.numeric(number, tz = "GMT")
    attr(x, "tzone") <- "GMT"
    as.POSIXct(x)
  }
  func.time_series_data <- function(data) {
    output <- list(outcome = NA, start = NA, interval = NA)
    if (TRUE %in% (is.na(data[,rv$entry[[2]]]))) {
      output <- list(outcome = "Unsuccessful", start = NA, interval = NA, date = "There cannot be missing values in the date. Please try again with corrected data or another date field.")
    } else if (rv$entry[[3]] != "") {
      if (TRUE %in% (is.na(data[,rv$entry[[3]]]))) {
        output <- list(outcome = "Unsuccessful", start = NA, interval = NA, date = "There cannot be missing values in the time. Please try again with corrected data or another time field.")
      }
    }
    if (is.na(output$outcome)) {
      if (rv$entry[[3]] != "") {
        date <- as.numeric(data[,rv$entry[[2]]]) + as.numeric(data[,rv$entry[[3]]])
      } else {
        date <- as.numeric(data[,rv$entry[[2]]])
      }
      difference <- sapply(2:length(date), function(x) {
        as.numeric(date[x]) - as.numeric(date[x-1])
      })
      average_difference <- mean(difference)
      output <- list(outcome = "Successful", start = date[1], interval = average_difference, date = date)
    }
    return(output)
  }
  func.determine_x_ticks <- function(lower_limit, upper_limit, interval){
    if (interval < 86400) {
      difference_in_minutes <- (upper_limit - lower_limit)/60
      # If difference is 12 hours or more, every 6 hours
      if (difference_in_minutes >= (12*60)) {
        lower_limit <- as.numeric(as.POSIXct(format(func.convert_to_date(lower_limit), "%Y-%m-%d"), tz="GMT"))
        x_ticks <- lower_limit + (3600 * c(0, 6, 12, 18, 24))
        x_labels <- c(paste0(format(func.convert_to_date(lower_limit), "%e"), "-", month.abb[as.numeric(format(func.convert_to_date(lower_limit), "%m"))], " ", c("00:00", "06:00", "12:00", "18:00")),
                      paste0(format(func.convert_to_date(lower_limit + 86400), "%e"), "-", month.abb[as.numeric(format(func.convert_to_date(lower_limit + 86400), "%m"))], " ", "00:00"))
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
        # If difference is 4 hours or more, every 2 hours
      } else if (difference_in_minutes >= (4*60)) {
        closest_even_hour <- as.numeric(format(func.convert_to_date(lower_limit), "%H"))
        closest_even_hour <- ifelse((closest_even_hour %% 2) == 1, closest_even_hour - 1, closest_even_hour - 2)
        if (closest_even_hour < 0) {
          date <- format(func.convert_to_date(lower_limit - 86400), "%Y-%m-%d")
          closest_even_hour <- (24 + closest_even_hour)
        } else {
          date <- format(func.convert_to_date(lower_limit), "%Y-%m-%d")
        }
        lower_limit <- as.numeric(as.POSIXct(paste0(date, " ", formatC(closest_even_hour, width = 2, flag = "0"), ":00:00"), tz="GMT"))
        x_ticks <- lower_limit + (3600 * c(0, 4, 8, 12, 16, 20))
        x_labels <- sapply(x_ticks, function(x) {
          paste0(format(func.convert_to_date(x), "%e"), "-", month.abb[as.numeric(format(func.convert_to_date(x), "%m"))], " ", format(func.convert_to_date(x), "%H"), ":00")
        })
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
        # If difference is 1 hour or more, every hour
      } else if (difference_in_minutes >= 60) {
        closest_hour <- as.numeric(format(func.convert_to_date(lower_limit), "%H")) - 1
        if (closest_hour < 0) {
          date <- format(func.convert_to_date(lower_limit - 86400), "%Y-%m-%d")
          closest_hour <- (24 + closest_hour)
        } else {
          date <- format(func.convert_to_date(lower_limit), "%Y-%m-%d")
        }
        lower_limit <- as.numeric(as.POSIXct(paste0(date, " ", formatC(closest_hour, width = 2, flag = "0"), ":00:00"), tz="GMT"))
        x_ticks <- lower_limit + (3600 * c(0, 1, 2, 3, 4, 5))
        x_labels <- sapply(x_ticks, function(x) {
          paste0(format(func.convert_to_date(x), "%e"), "-", month.abb[as.numeric(format(func.convert_to_date(x), "%m"))], " ", format(func.convert_to_date(x), "%H"), ":00")
        })
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
        # If difference is 10 minutes or more, every 10 minutes
      } else if (difference_in_minutes >= 10) {
        closest_10_minutes <- (round(as.numeric(format(func.convert_to_date(lower_limit), "%M")),-1) - 10)
        if (closest_10_minutes < 0) {
          date_hour <- format(func.convert_to_date(lower_limit - 3600), "%Y-%m-%d %H")
          closest_10_minutes <- (60 + closest_10_minutes)
        } else {
          date_hour <- format(func.convert_to_date(lower_limit), "%Y-%m-%d %H")
        }
        lower_limit <- as.numeric(as.POSIXct(paste0(date_hour, ":", formatC(closest_10_minutes, width = 2, flag = "0"), ":00"), tz="GMT"))
        x_ticks <- lower_limit + (60 * c(0, 10, 20, 30, 40, 50, 60, 70))
        x_labels <- sapply(x_ticks, function(x) {format(func.convert_to_date(x), "%H:%M")})
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
        # If difference is 1 minute or more, every 2 minutes
      } else if (difference_in_minutes >= 1) {
        closest_even_minutes <- round(as.numeric(format(func.convert_to_date(lower_limit), "%M")))
        closest_even_minutes <- ifelse((closest_even_minutes %% 2) == 1, closest_even_minutes - 1, closest_even_minutes - 2)
        if (closest_even_minutes < 0) {
          date_hour <- format(func.convert_to_date(lower_limit - 3600), "%Y-%m-%d %H")
          closest_even_minutes <- (60 + closest_even_minutes)
        } else {
          date_hour <- format(func.convert_to_date(lower_limit), "%Y-%m-%d %H")
        }
        lower_limit <- as.numeric(as.POSIXct(paste0(date_hour, ":", formatC(closest_even_minutes, width = 2, flag = "0"), ":00"), tz="GMT"))
        x_ticks <- lower_limit + (60 * c(0, 2, 4, 6, 8, 10, 12))
        x_labels <- sapply(x_ticks, function(x) {format(func.convert_to_date(x), "%H:%M")})
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
        # If difference is 10 seconds or more, every 10 seconds
      } else if (interval >= 10) {
        closest_10_seconds <- (round(as.numeric(format(func.convert_to_date(lower_limit), "%S")),-1) - 10)
        if (closest_10_seconds < 0) {
          date_hour_min <- format(func.convert_to_date(lower_limit - 60), "%Y-%m-%d %H:%M")
          closest_10_seconds <- (60 + closest_10_seconds)
        } else {
          date_hour_min <- format(func.convert_to_date(lower_limit), "%Y-%m-%d %H:%M")
        }
        lower_limit <- as.numeric(as.POSIXct(paste0(date_hour_min, ":", formatC(closest_10_seconds, width = 2, flag = "0")), tz="GMT"))
        x_ticks <- lower_limit + c(0, 10, 20, 30, 40, 50, 60, 70)
        x_labels <- sapply(x_ticks, function(x) {paste0(format(func.convert_to_date(x), "%M:%S"), " min:sec")})
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
      } else {
        closest_second <- as.numeric(format(func.convert_to_date(lower_limit), "%S")) - 1
        if (closest_second < 0) {
          date_hour_min <- format(func.convert_to_date(lower_limit - 60), "%Y-%m-%d %H:%M")
          closest_second <- (60 + closest_second)
        } else {
          date_hour_min <- format(func.convert_to_date(lower_limit), "%Y-%m-%d %H:%M")
        }
        lower_limit <- as.numeric(as.POSIXct(paste0(date_hour_min, ":", formatC(closest_second, width = 2, flag = "0")), tz="GMT"))
        x_ticks <- lower_limit + c(0, 2, 4, 6, 8, 10, 12)
        x_labels <- sapply(x_ticks, function(x) {paste0(format(func.convert_to_date(x), "%M:%S"), " min:sec")})
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
      }
    } else {
      difference_in_days <- (upper_limit - lower_limit)/86400
      # If number of days > 4 years, it should be years
      if (difference_in_days > (4*365.25)) {
        lower_limit <- as.numeric(as.character(format(func.convert_to_date(lower_limit), "%Y")))
        upper_limit <- as.numeric(as.character(format(func.convert_to_date(upper_limit), "%Y"))) + 1
        x_ticks <- pretty(lower_limit:upper_limit, 4)
        x_labels <- as.character(x_ticks)
        x_ticks <- as.numeric(as.POSIXct(paste0(x_ticks, "-01-01 00:00:00 GMT")))
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
        # If it is between 2 years and 4 years, annual
      } else if (difference_in_days > (2 * 365.25)) {
        lower_limit <- as.numeric(as.character(format(func.convert_to_date(lower_limit), "%Y")))
        upper_limit <- lower_limit + 5
        x_ticks <- lower_limit:upper_limit
        x_labels <- as.character(x_ticks)
        x_ticks <- as.numeric(as.POSIXct(paste0(x_ticks, "-01-01 00:00:00 GMT")))
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
        # If it is between 1 year and 2 years, 6 monthly
      } else if (difference_in_days > 365.25) {
        lower_limit <- as.numeric(as.character(format(func.convert_to_date(lower_limit), "%Y")))
        upper_limit <- as.numeric(as.character(format(func.convert_to_date(upper_limit), "%Y"))) + 1
        x_ticks <- as.numeric(as.vector(sapply(1:length(lower_limit:upper_limit), function(x) {as.POSIXct(paste0((lower_limit:upper_limit)[x], c("-01-01 00:00:00 GMT", "-07-01 00:00:00 GMT")))})))
        x_labels <- as.vector(sapply(1:length(lower_limit:upper_limit), function(x) {paste0((lower_limit:upper_limit)[x], c(" 1st half", " 2nd half"))}))
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
        # If it is between 3 months and 1 year, quarterly
      } else if (difference_in_days > 90) {
        which_quarter <- quarters(func.convert_to_date(lower_limit))
        lower_limit <- as.numeric(as.character(format(func.convert_to_date(lower_limit), "%Y")))
        x_ticks_list <- c(paste0(lower_limit, c("-01-01 GMT", "-04-01 GMT", "-07-01 GMT", "-10-01 GMT")),
                          paste0((lower_limit+1), c("-01-01 GMT", "-04-01 GMT", "-07-01 GMT", "-10-01 GMT")),
                          paste0((lower_limit+2), c("-01-01 GMT", "-04-01 GMT", "-07-01 GMT", "-10-01 GMT"))
        )
        x_labels_list <- c(paste0(lower_limit, c(" Q1", " Q2", " Q3", " Q4")),
                           paste0((lower_limit+1), c(" Q1", " Q2", " Q3", " Q4")),
                           paste0((lower_limit+2), c(" Q1", " Q2", " Q3", " Q4"))
        )
        x_ticks <- as.numeric(as.vector(sapply(x_ticks_list[as.numeric(substr(which_quarter,2,2)):(as.numeric(substr(which_quarter,2,2))+ 6)],as.POSIXct)))
        x_labels <- x_labels_list[as.numeric(substr(which_quarter,2,2)):(as.numeric(substr(which_quarter,2,2))+ 6)]
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
        # If it is between 1 month and 3 months, months
      } else if (difference_in_days > 30) {
        which_month <- match(months(func.convert_to_date(lower_limit)), month.name)
        lower_limit <- as.numeric(as.character(format(func.convert_to_date(lower_limit), "%Y")))
        x_ticks_list <- c(paste0(lower_limit, "-", formatC(1:12, width = 2, flag = "0"), "-01 GMT"), paste0((lower_limit + 1), "-", formatC(1:12, digits = 1, flag = "0"),"-01 GMT"))
        x_labels_list <- c(paste0(month.abb, " ", lower_limit), paste0(month.abb, " ", (lower_limit + 1)))
        x_ticks <- as.numeric(as.vector(sapply(x_ticks_list[which_month:(which_month + 3)],as.POSIXct)))
        x_labels <- x_labels_list[which_month:(which_month + 3)]
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
        # If it is between 1 week and 30 days, specific days starting from 1
      } else if (difference_in_days > 7) {
        x_ticks <- seq(from = lower_limit, to = upper_limit, by = (86400 *7))
        x_ticks <- c(x_ticks, max(x_ticks) + (86400 *7))
        x_ticks <- sapply(x_ticks, func.convert_to_date)
        x_labels <- as.character(as.POSIXct(x_ticks, tz = "GMT"))
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
      } else {
        x_ticks <- seq(from = lower_limit, to= upper_limit,  by = 86400)
        x_labels <- as.character(as.POSIXct(x_ticks, tz = "GMT"))
        lower_limit <- min(x_ticks)
        upper_limit <- max(x_ticks)
      }
    }
    output <- list(x_ticks = x_ticks, x_labels = x_labels, lower_limit = lower_limit, upper_limit = upper_limit)
    return(output)
  }
  if (TRUE %in% (rv$entry[[4]] != "")) {
    xreg <- data.matrix(rv$import_data$data[,rv$entry[[4]]])
    colnames(xreg) <- rv$entry[[4]]
    xreg_future <- cbind.data.frame(lapply(1:length(rv$entry[[4]]), function(y) {
      if (is.numeric(rv$import_data$data[,rv$entry[[4]][y]])) {
        output <- mean(xreg[,y], na.rm = TRUE)
      } else {
        frequencies <- as.data.frame(table(as.factor(rv$import_data$data[,rv$entry[[4]][y]])))
        output <- frequencies$Var1[which.max(frequencies$Freq)]
      }
    }))
    colnames(xreg_future) <- rv$entry[[4]]
    xreg_future <- do.call(rbind.data.frame, lapply(1:as.numeric(rv$entry[[6]]), function(x) {xreg_future[1,]}))
    xreg_future <- data.matrix(xreg_future)
    colnames(xreg_future) <- rv$entry[[4]]
  }
  processed_data <- func.recognise_date(data = rv$import_data$data)
  data_converted <- lapply(1:ncol(rv$import_data$data), function(y) {
    if (colnames(rv$import_data$data)[y] == rv$entry[[2]]) {
      column_data <- as.character(rv$import_data$data[,y])
      column_data[! is.na(column_data)] <- processed_data[[y]][[2]]
    } else {
      column_data <- rv$import_data$data[,y]
    }
    return(column_data)
  })
  names(data_converted) <- colnames(rv$import_data$data)
  data_converted <- do.call(cbind.data.frame, data_converted)
  if (rv$entry[[3]] != "") {
    data_converted <- lapply(1:ncol(rv$import_data$data), function(y) {
      if (colnames(data_converted)[y] == rv$entry[[3]]) {
        column_data <- as.character(data_converted[,y])
        column_data[! is.na(column_data)] <- as.POSIXct(paste0("1970-01-01 ",column_data[! is.na(column_data)], tz = "GMT"))
      } else {
        column_data <- data_converted[,y]
      }
      return(column_data)
    })
    names(data_converted) <- colnames(rv$import_data$data)
    data_converted <- do.call(cbind.data.frame, data_converted)
    data <- data_converted[,c(as.numeric(rv$entry[[1]]), rv$entry[[2]], rv$entry[[3]])]
    sort_order <- order(as.numeric(data[,rv$entry[[2]]]) + as.numeric(data[,rv$entry[[3]]]))
  } else {
    data <- data_converted[,c(rv$entry[[1]], rv$entry[[2]])]
    sort_order <- order(as.numeric(data[,rv$entry[[2]]]))
  }
  data <- data[order(sort_order), ]
  time_series <- func.time_series_data(data)
  if (time_series$outcome == "Successful") {
    if (length(rv$entry[[1]]) == 1) {
      data_for_analysis <-  data.frame(ts(data = data[,rv$entry[[1]]], start = func.convert_to_date(time_series$start), deltat = time_series$interval), check.names = FALSE)
    } else {
      data_for_analysis <-  ts(data = data[,rv$entry[[1]]], start = func.convert_to_date(time_series$start), deltat = time_series$interval)
    }
    if ("date_time" %in% colnames(data_for_analysis)) {
      colnames(data_for_analysis)[colnames(data_for_analysis) == "date_time"] <- "date_time_input"
    }
    data_for_plot <- cbind.data.frame(data_for_analysis, date_time = time_series$date)
    model_fit <- list()
    residuals <- list()
    model_forecast <- list()
    data_forecast <- list()
    geom_line_text <- list()
    geom_line_text_residuals <- list()
    geom_line_text_with_forecast <- list()
    number_of_predictions <- as.numeric(rv$entry[[6]])
    new_colours <- viridis((5*length(rv$entry[[1]])))
    if ((rv$entry[[5]] == "Univariate") | (length(rv$entry[[1]]) == 1)) {
      for (i in 1:length(rv$entry[[1]])) {
        if (TRUE %in% (rv$entry[[4]] != "")) {
          model_fit[[i]] <- auto.arima(data_for_analysis[,i], xreg = xreg)
          residuals[[i]] <- model_fit[[i]]$residuals
          model_forecast[[i]] <- forecast(object = model_fit[[i]], h = number_of_predictions, xreg = xreg_future)
        } else {
          model_fit[[i]] <- auto.arima(data_for_analysis[,i])
          residuals[[i]] <- model_fit[[i]]$residuals
          model_forecast[[i]] <- forecast(model_fit[[i]], number_of_predictions)
        }
        data_forecast[[i]] <- cbind.data.frame(date_time = seq(from = attr(model_forecast[[i]]$mean, "tsp")[1], to = attr(model_forecast[[i]]$mean, "tsp")[2], length.out = number_of_predictions),
                                               forecast_average = model_forecast[[i]]$mean,
                                               lower = model_forecast[[i]]$lower,
                                               upper = model_forecast[[i]]$upper
        )
        x_values <- sapply(data_for_plot$date_time, func.convert_to_date)
        graph_x_tick_details <- func.determine_x_ticks(lower_limit = min(time_series$date), upper_limit = max(time_series$date), interval = time_series$interval)
        x_values_2 <- sapply(data_forecast[[1]]$date_time, func.convert_to_date)
        graph_x_tick_details_2 <- func.determine_x_ticks(lower_limit = min(data_forecast[[1]]$date_time), upper_limit = max(data_forecast[[1]]$date_time), interval = time_series$interval)
        geom_line_text[[i]] <- paste0('      geom_line(aes(y=data[,rv$entry[[1]][', i, ']], colour = rv$entry[[1]][', i, '])) + \n')
        geom_line_text_with_forecast[[i]] <- paste0('
      geom_line(aes(y=as.numeric(data_forecast[[', i, ']][,2]), colour = new_colours[', (i-1)*3+1, '], linetype = "1"), linewidth = 1.1) +
      geom_line(aes(y=as.numeric(data_forecast[[', i, ']][,3]), colour = new_colours[', (i-1)*3+2, '], linetype = "3"), linewidth = 0.6) +
      geom_line(aes(y=as.numeric(data_forecast[[', i, ']][,4]), colour = new_colours[', (i-1)*3+3, '], linetype = "4"), linewidth = 0.7) +
      geom_line(aes(y=as.numeric(data_forecast[[', i, ']][,5]), colour = new_colours[', (i-1)*3+2, '], linetype = "3"), linewidth = 0.6) +
      geom_line(aes(y=as.numeric(data_forecast[[', i, ']][,6]), colour = new_colours[', (i-1)*3+3, '], linetype = "4"), linewidth = 0.7) +
      geom_ribbon(aes(ymin = as.numeric(data_forecast[[', i, ']][,4]), ymax = as.numeric(data_forecast[[', i, ']][,6])), fill = new_colours[', (i-1)*3+1, '], alpha = 0.1) +
                                                ')
        geom_line_text_residuals[[i]] <- paste0('      geom_line(aes(y=as.numeric(residuals[,', i, ']), colour = rv$entry[[1]][', i, '])) + \n')
      }
      geom_line_text <- do.call(paste0, geom_line_text)
      geom_line_text_residuals <- do.call(paste0, geom_line_text_residuals)
      geom_line_text_with_forecast <- do.call(paste0, geom_line_text_with_forecast)
      residuals <- do.call(cbind.data.frame, residuals)
      colnames(residuals) <- rv$entry[[1]]
      plot_text_residuals <- paste0('ggplot(residuals, aes(x=x_values)) + \n', geom_line_text_residuals,
                                    '      xlab("Time") + ylab("Value") + theme_classic() +
      scale_colour_viridis_d(name = "Group") +
      scale_x_continuous(limits = c(graph_x_tick_details$lower_limit, graph_x_tick_details$upper_limit),
                        breaks = graph_x_tick_details$x_ticks,
                        labels = graph_x_tick_details$x_labels
                        )')
    } else {
      season <- max(sapply(1:ncol(data_for_analysis), function(y) {findfrequency(data_for_analysis[,y])}))
      lag_order <- suppressWarnings(max(sapply(1:ncol(data_for_analysis), function(y) {tseries::adf.test(data_for_analysis[,y])$parameter})))
      differences <- max(sapply(1:ncol(data_for_analysis), function(y) {ndiffs(data_for_analysis[,y])}))
      if (season > 1) {
        differences_seasonal <- suppressWarnings(max(sapply(1:ncol(data_for_analysis), function(y) {nsdiffs(data_for_analysis[,y])})))
        if (TRUE %in% (rv$entry[[4]] != "")) {
          if (differences > 0) {
            if (differences_seasonal >0) {
              cointegration <- summary(ca.jo(diff(diff(data_for_analysis, differences = differences_seasonal),  lag = lag_order,  differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = diff(diff(xreg, differences = differences_seasonal),  lag = lag_order,  differences = differences), season = season))
            } else {
              cointegration <- summary(ca.jo(diff(data_for_analysis, lag = lag_order, differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = diff(xreg, lag = lag_order, differences = differences), season = season))
            }
            reject_r0 <- (cointegration@teststat[length(cointegration@teststat)] > cointegration@cval[length(cointegration@teststat),2])
            if (reject_r0 == TRUE) {
              r = cbind.data.frame(test_statistic = cointegration@teststat, critical_value_0.05 = cointegration@cval[,2])
              r$r <- (nrow(r)-1):0
              r <- r[order(r$r),]
              r$reject_hypothesis <- (r$test_statistic > r$critical_value_0.05)
              if (is.na(match(FALSE, r$reject_hypothesis))) {
                r <- max(r$r + 1)
              } else {
                r <- r$r[match(FALSE, r$reject_hypothesis)]
              }
              if (differences_seasonal >0) {
                model_fit <- try(vec2var(ca.jo(diff(diff(data_for_analysis, differences = differences_seasonal),  lag = lag_order,  differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = diff(diff(xreg, differences = differences_seasonal),  lag = lag_order,  differences = differences), season = season), r = r), silent = TRUE)
                if (str_detect(model_fit[[1]][1], "Error")) {
                  limits <-  substr(model_fit,  (str_locate(model_fit[[1]][1],"\\[")[1,1] + 1), (str_locate(model_fit[[1]][1],"\\]")[1,1] - 1))
                  r <- max(as.numeric(unlist(str_split(limits, ":"))))
                  model_fit <- try(vec2var(ca.jo(diff(diff(data_for_analysis, differences = differences_seasonal),  lag = lag_order,  differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = diff(diff(xreg, differences = differences_seasonal),  lag = lag_order,  differences = differences), season = season), r = r), silent = TRUE)
                }
              } else {
                model_fit <- try(vec2var(ca.jo(diff(data_for_analysis, lag = lag_order, differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = diff(xreg, lag = lag_order, differences = differences), season = season), r = r), silent = TRUE)
                if (str_detect(model_fit[[1]][1], "Error")) {
                  limits <-  substr(model_fit,  (str_locate(model_fit[[1]][1],"\\[")[1,1] + 1), (str_locate(model_fit[[1]][1],"\\]")[1,1] - 1))
                  r <- max(as.numeric(unlist(str_split(limits, ":"))))
                  model_fit <- try(vec2var(ca.jo(diff(data_for_analysis, lag = lag_order, differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = diff(xreg, lag = lag_order, differences = differences), season = season), r = r), silent = TRUE)
                }
              }
            } else {
              if (differences_seasonal > 0) {
                model_fit <- VAR(diff(diff(data_for_analysis, differences = differences_seasonal),  lag = lag_order,  differences = differences), p = lag_order, exogen = diff(diff(xreg, differences = differences_seasonal),  lag = lag_order,  differences = differences), season = season)
              } else {
                model_fit <- VAR(diff(data_for_analysis, lag = lag_order, differences = differences), p = lag_order, exogen = diff(xreg, lag = lag_order, differences = differences), season = season)
              }
            }
          } else {
            if (differences_seasonal > 0) {
              cointegration <- summary(ca.jo(diff(data_for_analysis, differences = differences_seasonal), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = diff(xreg, differences = differences_seasonal), season = season))
            } else {
              cointegration <- summary(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = xreg, season = season))
            }
            reject_r0 <- (cointegration@teststat[length(cointegration@teststat)] > cointegration@cval[length(cointegration@teststat),2])
            if (reject_r0 == TRUE) {
              r = cbind.data.frame(test_statistic = cointegration@teststat, critical_value_0.05 = cointegration@cval[,2])
              r$r <- (nrow(r)-1):0
              r <- r[order(r$r),]
              r$reject_hypothesis <- (r$test_statistic > r$critical_value_0.05)
              if (is.na(match(FALSE, r$reject_hypothesis))) {
                r <- max(r$r + 1)
              } else {
                r <- r$r[match(FALSE, r$reject_hypothesis)]
              }
              if (differences_seasonal >0) {
                model_fit <- try(vec2var(ca.jo(diff(data_for_analysis, differences = differences_seasonal), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = diff(xreg, differences = differences_seasonal), season = season), r = r), silent = TRUE)
                if (str_detect(model_fit[[1]][1], "Error")) {
                  limits <-  substr(model_fit,  (str_locate(model_fit[[1]][1],"\\[")[1,1] + 1), (str_locate(model_fit[[1]][1],"\\]")[1,1] - 1))
                  r <- max(as.numeric(unlist(str_split(limits, ":"))))
                  model_fit <- try(vec2var(ca.jo(diff(data_for_analysis, differences = differences_seasonal), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = diff(xreg, differences = differences_seasonal), season = season), r = r), silent = TRUE)
                }
              } else {
                model_fit <- try(vec2var(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = xreg, season = season), r = r), silent = TRUE)
                if (str_detect(model_fit[[1]][1], "Error")) {
                  limits <-  substr(model_fit,  (str_locate(model_fit[[1]][1],"\\[")[1,1] + 1), (str_locate(model_fit[[1]][1],"\\]")[1,1] - 1))
                  r <- max(as.numeric(unlist(str_split(limits, ":"))))
                  model_fit <- try(vec2var(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = xreg, season = season), r = r), silent = TRUE)
                }
              }
            } else {
              if (differences_seasonal > 0) {
                model_fit <- VAR(diff(data_for_analysis, differences = differences_seasonal), p = lag_order, exogen = diff(xreg, differences = differences_seasonal), season = season)
              } else {
                model_fit <- VAR(data_for_analysis, p = lag_order, exogen = xreg, season = season)
              }
            }
          }
        } else {
          if (differences > 0) {
            if (differences_seasonal > 0) {
              cointegration <- summary(ca.jo(diff(diff(data_for_analysis, differences = differences_seasonal),  lag = lag_order,  differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", season = season))
            } else {
              cointegration <- summary(ca.jo(diff(data_for_analysis, lag = lag_order, differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", season = season))
            }
            reject_r0 <- (cointegration@teststat[length(cointegration@teststat)] > cointegration@cval[length(cointegration@teststat),2])
            if (reject_r0 == TRUE) {
              r = cbind.data.frame(test_statistic = cointegration@teststat, critical_value_0.05 = cointegration@cval[,2])
              r$r <- (nrow(r)-1):0
              r <- r[order(r$r),]
              r$reject_hypothesis <- (r$test_statistic > r$critical_value_0.05)
              if (is.na(match(FALSE, r$reject_hypothesis))) {
                r <- max(r$r + 1)
              } else {
                r <- r$r[match(FALSE, r$reject_hypothesis)]
              }
              if (differences_seasonal >0) {
                model_fit <- try(vec2var(ca.jo(diff(diff(data_for_analysis, differences = differences_seasonal),  lag = lag_order,  differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", season = season), r = r), silent = TRUE)
                if (str_detect(model_fit[[1]][1], "Error")) {
                  limits <-  substr(model_fit,  (str_locate(model_fit[[1]][1],"\\[")[1,1] + 1), (str_locate(model_fit[[1]][1],"\\]")[1,1] - 1))
                  r <- max(as.numeric(unlist(str_split(limits, ":"))))
                  model_fit <- try(vec2var(ca.jo(diff(diff(data_for_analysis, differences = differences_seasonal),  lag = lag_order,  differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", season = season), r = r), silent = TRUE)
                }
              } else {
                model_fit <- try(vec2var(ca.jo(diff(data_for_analysis, lag = lag_order, differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", season = season), r = r), silent = TRUE)
                if (str_detect(model_fit[[1]][1], "Error")) {
                  limits <-  substr(model_fit,  (str_locate(model_fit[[1]][1],"\\[")[1,1] + 1), (str_locate(model_fit[[1]][1],"\\]")[1,1] - 1))
                  r <- max(as.numeric(unlist(str_split(limits, ":"))))
                  model_fit <- try(vec2var(ca.jo(diff(data_for_analysis, lag = lag_order, differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", season = season), r = r), silent = TRUE)
                }
              }
            } else {
              if (differences_seasonal > 0) {
                model_fit <- VAR(diff(diff(data_for_analysis, differences = differences_seasonal),  lag = lag_order,  differences = differences), p = lag_order, season = season)
              } else {
                model_fit <- VAR(diff(data_for_analysis, lag = lag_order, differences = differences), p = lag_order, season = season)
              }
            }
          } else {
            if (differences_seasonal > 0) {
              cointegration <- summary(ca.jo(diff(data_for_analysis, differences = differences_seasonal), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", season = season))
            } else {
              cointegration <- summary(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", season = season))
            }
            reject_r0 <- (cointegration@teststat[length(cointegration@teststat)] > cointegration@cval[length(cointegration@teststat),2])
            if (reject_r0 == TRUE) {
              r = cbind.data.frame(test_statistic = cointegration@teststat, critical_value_0.05 = cointegration@cval[,2])
              r$r <- (nrow(r)-1):0
              r <- r[order(r$r),]
              r$reject_hypothesis <- (r$test_statistic > r$critical_value_0.05)
              if (is.na(match(FALSE, r$reject_hypothesis))) {
                r <- max(r$r + 1)
              } else {
                r <- r$r[match(FALSE, r$reject_hypothesis)]
              }
              if (differences_seasonal >0) {
                model_fit <- try(vec2var(ca.jo(diff(data_for_analysis, differences = differences_seasonal), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", season = season), r = r), silent = TRUE)
                if (str_detect(model_fit[[1]][1], "Error")) {
                  limits <-  substr(model_fit,  (str_locate(model_fit[[1]][1],"\\[")[1,1] + 1), (str_locate(model_fit[[1]][1],"\\]")[1,1] - 1))
                  r <- max(as.numeric(unlist(str_split(limits, ":"))))
                  model_fit <- try(vec2var(ca.jo(diff(data_for_analysis, differences = differences_seasonal), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", season = season), r = r), silent = TRUE)
                }
              } else {
                model_fit <- try(vec2var(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", season = season), r = r), silent = TRUE)
                if (str_detect(model_fit[[1]][1], "Error")) {
                  limits <-  substr(model_fit,  (str_locate(model_fit[[1]][1],"\\[")[1,1] + 1), (str_locate(model_fit[[1]][1],"\\]")[1,1] - 1))
                  r <- max(as.numeric(unlist(str_split(limits, ":"))))
                  model_fit <- try(vec2var(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", season = season), r = r), silent = TRUE)
                }
              }
            } else {
              if (differences_seasonal > 0) {
                model_fit <- VAR(diff(data_for_analysis, differences = differences_seasonal), p = lag_order, season = season)
              } else {
                model_fit <- VAR(data_for_analysis, p = lag_order, season = season)
              }
            }
          }
        }
      } else {
        if (TRUE %in% (rv$entry[[4]] != "")) {
          if (differences > 0) {
            cointegration <- summary(ca.jo(diff(data_for_analysis, lag = lag_order, differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = diff(xreg, lag = lag_order, differences = differences)))
            reject_r0 <- (cointegration@teststat[length(cointegration@teststat)] > cointegration@cval[length(cointegration@teststat),2])
            if (reject_r0 == TRUE) {
              r = cbind.data.frame(test_statistic = cointegration@teststat, critical_value_0.05 = cointegration@cval[,2])
              r$r <- (nrow(r)-1):0
              r <- r[order(r$r),]
              r$reject_hypothesis <- (r$test_statistic > r$critical_value_0.05)
              if (is.na(match(FALSE, r$reject_hypothesis))) {
                r <- max(r$r + 1)
              } else {
                r <- r$r[match(FALSE, r$reject_hypothesis)]
              }
              model_fit <- try(vec2var(ca.jo(diff(data_for_analysis, lag = lag_order, differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = diff(xreg, lag = lag_order, differences = differences)), r = r), silent = TRUE)
              if (str_detect(model_fit[[1]][1], "Error")) {
                limits <-  substr(model_fit,  (str_locate(model_fit[[1]][1],"\\[")[1,1] + 1), (str_locate(model_fit[[1]][1],"\\]")[1,1] - 1))
                r <- max(as.numeric(unlist(str_split(limits, ":"))))
                model_fit <- try(vec2var(ca.jo(diff(data_for_analysis, lag = lag_order, differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = diff(xreg, lag = lag_order, differences = differences)), r = r), silent = TRUE)
              }
            } else {
              model_fit <- VAR(diff(data_for_analysis, lag = lag_order, differences = differences), p = lag_order, exogen = diff(xreg, lag = lag_order, differences = differences))
            }
          } else {
            cointegration <- summary(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = xreg))
            reject_r0 <- (cointegration@teststat[length(cointegration@teststat)] > cointegration@cval[length(cointegration@teststat),2])
            if (reject_r0 == TRUE) {
              r = cbind.data.frame(test_statistic = cointegration@teststat, critical_value_0.05 = cointegration@cval[,2])
              r$r <- (nrow(r)-1):0
              r <- r[order(r$r),]
              r$reject_hypothesis <- (r$test_statistic > r$critical_value_0.05)
              if (is.na(match(FALSE, r$reject_hypothesis))) {
                r <- max(r$r + 1)
              } else {
                r <- r$r[match(FALSE, r$reject_hypothesis)]
              }
              model_fit <- try(vec2var(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = xreg), r = r), silent = TRUE)
              if (str_detect(model_fit[[1]][1], "Error")) {
                limits <-  substr(model_fit,  (str_locate(model_fit[[1]][1],"\\[")[1,1] + 1), (str_locate(model_fit[[1]][1],"\\]")[1,1] - 1))
                r <- max(as.numeric(unlist(str_split(limits, ":"))))
                model_fit <- try(vec2var(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory", dumvar = xreg), r = r), silent = TRUE)
              }
            } else {
              model_fit <- VAR(data_for_analysis, p = lag_order, exogen = xreg)
            }
          }
        } else {
          if (differences > 0) {
            cointegration <- summary(ca.jo(diff(data_for_analysis, lag = lag_order, differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory"))
            reject_r0 <- (cointegration@teststat[length(cointegration@teststat)] > cointegration@cval[length(cointegration@teststat),2])
            if (reject_r0 == TRUE) {
              r = cbind.data.frame(test_statistic = cointegration@teststat, critical_value_0.05 = cointegration@cval[,2])
              r$r <- (nrow(r)-1):0
              r <- r[order(r$r),]
              r$reject_hypothesis <- (r$test_statistic > r$critical_value_0.05)
              if (is.na(match(FALSE, r$reject_hypothesis))) {
                r <- max(r$r + 1)
              } else {
                r <- r$r[match(FALSE, r$reject_hypothesis)]
              }
              model_fit <- try(vec2var(ca.jo(diff(data_for_analysis, lag = lag_order, differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory"), r = r), silent = TRUE)
              if (str_detect(model_fit[[1]][1], "Error")) {
                limits <-  substr(model_fit,  (str_locate(model_fit[[1]][1],"\\[")[1,1] + 1), (str_locate(model_fit[[1]][1],"\\]")[1,1] - 1))
                r <- max(as.numeric(unlist(str_split(limits, ":"))))
                model_fit <- try(vec2var(ca.jo(diff(data_for_analysis, lag = lag_order, differences = differences), type = "eigen", ecdet = "const", K = lag_order, spec = "transitory"), r = r), silent = TRUE)
              }
            } else {
              model_fit <- VAR(diff(data_for_analysis, lag = lag_order, differences = differences), p = lag_order)
            }
          } else {
            cointegration <- summary(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory"))
            reject_r0 <- (cointegration@teststat[length(cointegration@teststat)] > cointegration@cval[length(cointegration@teststat),2])
            if (reject_r0 == TRUE) {
              r = cbind.data.frame(test_statistic = cointegration@teststat, critical_value_0.05 = cointegration@cval[,2])
              r$r <- (nrow(r)-1):0
              r <- r[order(r$r),]
              r$reject_hypothesis <- (r$test_statistic > r$critical_value_0.05)
              if (is.na(match(FALSE, r$reject_hypothesis))) {
                r <- max(r$r + 1)
              } else {
                r <- r$r[match(FALSE, r$reject_hypothesis)]
              }
              model_fit <- try(vec2var(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory"), r = r), silent = TRUE)
              if (str_detect(model_fit[[1]][1], "Error")) {
                limits <-  substr(model_fit,  (str_locate(model_fit[[1]][1],"\\[")[1,1] + 1), (str_locate(model_fit[[1]][1],"\\]")[1,1] - 1))
                r <- max(as.numeric(unlist(str_split(limits, ":"))))
                model_fit <- try(vec2var(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory"), r = r), silent = TRUE)
              }
              model_fit <- vec2var(ca.jo(data_for_analysis, type = "eigen", ecdet = "const", K = lag_order, spec = "transitory"), r = r)
            } else {
              model_fit <- VAR(data_for_analysis, p = lag_order)
            }
          }
        }
      }
      residuals <- resid(model_fit)
      time_series_details <- attr(model_fit$vecm@x, "tsp")
      geom_line_text_residuals <- do.call(paste0, lapply(1:length(rv$entry[[1]]), function(x) {paste0('geom_line(aes(y=residuals[,',x,'], colour = rv$entry[[1]][',x,'])) + \n')}))
      plot_text_residuals <- paste0('ggplot(residuals, aes(x=1:nrow(residuals))) + \n', geom_line_text_residuals,
                                    '      xlab("Time") + ylab("Value") + theme_classic() + theme(axis.text = element_blank()) +
      scale_colour_viridis_d(name = "Group")')
      geom_line_text <- do.call(paste0, lapply(1:length(rv$entry[[1]]), function(x) {paste0('geom_line(aes(y=data[,rv$entry[[1]][',x,']], colour = rv$entry[[1]][',x,'])) + \n')}))
      x_values <- sapply(data_for_plot$date_time, func.convert_to_date)
      graph_x_tick_details <- func.determine_x_ticks(lower_limit = min(time_series$date), upper_limit = max(time_series$date), interval = time_series$interval)
      x_values_2 <- sapply(seq(from = (time_series_details[2] + 1/time_series_details[3]), by = (1/time_series_details[3]), length.out = number_of_predictions), func.convert_to_date)
      graph_x_tick_details_2 <- func.determine_x_ticks(lower_limit = (time_series_details[2] + 1/time_series_details[3]), upper_limit = (time_series_details[2] + 1/time_series_details[3] + number_of_predictions/time_series_details[3]), interval = 1/time_series_details[3])
      if (TRUE %in% (rv$entry[[4]] != "")) {
        model_forecast_80 <- predict(model_fit, n.ahead = number_of_predictions, ci = 0.80, dumvar = xreg_future)
        model_forecast_95 <- predict(model_fit, n.ahead = number_of_predictions, ci = 0.95, dumvar = xreg_future)
      } else {
        model_forecast_80 <- predict(model_fit, n.ahead = number_of_predictions, ci = 0.80)
        model_forecast_95 <- predict(model_fit, n.ahead = number_of_predictions, ci = 0.95)
      }
      for (i in 1:length(rv$entry[[1]])) {
        data_forecast[[i]] <- cbind.data.frame(date_time = seq(from = (time_series_details[2] + 1/time_series_details[3]), by = (1/time_series_details[3]), length.out = number_of_predictions),
                                               forecast_average = model_forecast_80$fcst[[i]][,"fcst"],
                                               `lower.80%` = model_forecast_80$fcst[[i]][,"lower"],
                                               `lower.95%` = model_forecast_95$fcst[[i]][,"lower"],
                                               `upper.80%` = model_forecast_80$fcst[[i]][,"upper"],
                                               `upper.95%` = model_forecast_80$fcst[[i]][,"upper"]
        )
        geom_line_text_with_forecast[[i]] <- paste0('
      geom_line(aes(y=as.numeric(data_forecast[[', i, ']][,2]), colour = new_colours[', (i-1)*3+1, '], linetype = "1"), linewidth = 1.1) +
      geom_line(aes(y=as.numeric(data_forecast[[', i, ']][,3]), colour = new_colours[', (i-1)*3+2, '], linetype = "3"), linewidth = 0.6) +
      geom_line(aes(y=as.numeric(data_forecast[[', i, ']][,4]), colour = new_colours[', (i-1)*3+3, '], linetype = "4"), linewidth = 0.7) +
      geom_line(aes(y=as.numeric(data_forecast[[', i, ']][,5]), colour = new_colours[', (i-1)*3+2, '], linetype = "3"), linewidth = 0.6) +
      geom_line(aes(y=as.numeric(data_forecast[[', i, ']][,6]), colour = new_colours[', (i-1)*3+3, '], linetype = "4"), linewidth = 0.7) +
      geom_ribbon(aes(ymin = as.numeric(data_forecast[[', i, ']][,4]), ymax = as.numeric(data_forecast[[', i, ']][,6])), fill = new_colours[', (i-1)*3+1, '], alpha = 0.1) +
                                                ')
      }
      geom_line_text_with_forecast <- do.call(paste0, geom_line_text_with_forecast)
    }
    plot_text <- paste0('ggplot(data, aes(x=x_values)) + \n', geom_line_text,
                        '      xlab("Time") + ylab("Value") + theme_classic() +
      scale_colour_viridis_d(name = "Group") +
      scale_x_continuous(limits = c(graph_x_tick_details$lower_limit, graph_x_tick_details$upper_limit),
                        breaks = graph_x_tick_details$x_ticks,
                        labels = graph_x_tick_details$x_labels
                        )')
    plot_text_with_forecast <- paste0('ggplot(data_forecast[[1]], aes(x=x_values_2)) + \n', geom_line_text_with_forecast,
                                      '      xlab("Time") + ylab("Value") + theme_classic() +
      scale_x_continuous(limits = c(graph_x_tick_details_2$lower_limit, graph_x_tick_details_2$upper_limit),
                        breaks = graph_x_tick_details_2$x_ticks,
                        labels = graph_x_tick_details_2$x_labels
                        ) +
      scale_linetype_manual(name = "Line type",
                          values = c(1, 4, 3),
                          labels = c("Average", "80% limit", "95% limit")
  ) +
      scale_colour_manual(name = "Colour",
                          values = new_colours[1:(3*length(rv$entry[[1]]))],
                          labels = as.vector(sapply(1:length(rv$entry[[1]]), function(x) {paste0(rv$entry[[1]][x], ": ", c("Average", "80% limit", "95% limit"))}))
  )')
    time_series_plot <- eval(parse(text = plot_text))
    time_series_plot_with_forecast <- eval(parse(text = plot_text_with_forecast))
    residuals_plot <- eval(parse(text = plot_text_residuals))
    acf_plots <- lapply(1:length(rv$entry[[1]]), function(x) {
      temp <- acf(residuals[!is.na(residuals[,x]),x], plot = FALSE)
      acf <- cbind.data.frame(Lag = temp$lag/time_series$interval, ACF = temp$acf)
      acf <- acf[2:nrow(acf),]
      ggplot(data = acf, aes(x = Lag, y = ACF)) + geom_segment(yend = 0) + scale_y_continuous(limits = c(-ceiling(max(abs(acf$ACF))*10)/10,ceiling(max(abs(acf$ACF))*10)/10)) + scale_x_continuous(expand = c(0,0)) + theme(axis.line.y = element_line(colour = "black"), panel.background = element_blank(), plot.title = element_text(hjust = 0.5)) + geom_hline(yintercept = 0) + ggtitle(rv$entry[[1]][x])
    })
    acf_plots <- eval(parse(text = paste0('plot_grid(', paste0('acf_plots[[',1:length(rv$entry[[1]]),']]', collapse = ", "),', ncol = 1) + theme(plot.background = element_rect(fill = "white", colour = NA))')))
    plot_title <- "Time series analysis"
    plot_title_combined <- ggdraw() + draw_label("Time series analysis", color="darkgreen", size=14, fontface="bold", hjust = 0.5) + theme(plot.margin = margin(0, 0, 0, 7))
    plots_list <- c("time_series_plot", "time_series_plot_with_forecast", "residuals_plot", "acf_plots")
    plots_list <- sapply(1:length(plots_list), function(x){
      eval(parse(text = paste0(
        "suppressWarnings(suppressMessages(try(ggsave(filename = paste0(rv$StorageFolder,'/AN', formatC((length(rv$plan) + 1), width = 4, format = 'd', flag = 0), '_',
                                                                   substr(str_replace_all(plot_title, '[^[:alnum:]]', '_'), 1, 80) ,'_','",plots_list[x],"','.png'),
                                                 plot = ",plots_list[x],"), silent = TRUE)))"
      )))
    })
    plots_list_display <- paste0(rv$StorageFolder,'/AN', formatC((length(rv$plan) + 1), width = 4, format = 'd', flag = 0), '_',
                                 substr(str_replace_all(plot_title, '[^[:alnum:]]', '_'), 1, 80) ,'_','composite_plot','.png')
    composite_plot <- suppressWarnings(suppressMessages(plot_grid(plot_title_combined, plot_grid(time_series_plot, time_series_plot_with_forecast, residuals_plot, acf_plots, ncol = 2),
                                                                  ncol=1, rel_heights = c(0.1, 3)) + theme(plot.background = element_rect(fill = "white", colour = NA))))
    prediction_date_time <- sapply(data_forecast[[1]]$date_time, function(x) {as.character(func.convert_to_date(x))})
    data_forecast <- do.call(cbind.data.frame, lapply(1:length(rv$entry[[1]]), function(x) {
      data_forecast_each_outcome <- data_forecast[[x]][2:6]
      colnames(data_forecast_each_outcome) <- paste0(rv$entry[[1]][x], "_", c("forecast_average", "lower_80%", "lower_95%", "upper_80%", "upper_95%"))
      return(data_forecast_each_outcome)
    }))
    data_forecast_display <- cbind.data.frame(prediction_date_time, lapply(1:ncol(data_forecast), function(y) {round(data_forecast[,y], 4)}))
    colnames(data_forecast_display) <- c("prediction_date_time",colnames(data_forecast))
    data_forecast <- cbind.data.frame(prediction_date_time, data_forecast)
    results <- data.frame(
      `Analysis number` = paste0("AN",formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0)),
      `Analysis type` = rv$first_menu_choice,
      `Analysis outcome` = "Successful",
      check.names = FALSE
    )
    results_display <- results
    if ((rv$entry[[5]] == "Univariate") | (length(rv$entry[[1]]) == 1)) {
      regression_coefficients <- cbind.data.frame(
        coefficients = names(model_fit[[1]]$coef),
        do.call(cbind.data.frame, lapply(1:length(rv$entry[[1]]), function(x) {
          regression_coefficients_each_outcome <- cbind.data.frame(
            point_estimate = model_fit[[x]]$coef,
            se = sapply(1:length(model_fit[[x]]$coef), function(z) {model_fit[[x]]$var.coef[z,z]})^0.5
          )
          colnames(regression_coefficients_each_outcome) <- paste0(rv$entry[[1]][x], "_", colnames(regression_coefficients_each_outcome))
          return(regression_coefficients_each_outcome)
        }))
      )
      regression_coefficients_display <- cbind.data.frame(
        coefficients = names(model_fit[[1]]$coef),
        do.call(cbind.data.frame, lapply(1:length(rv$entry[[1]]), function(x) {
          regression_coefficients_each_outcome <- cbind.data.frame(
            point_estimate = round(model_fit[[x]]$coef,4),
            se = round(sapply(1:length(model_fit[[x]]$coef), function(z) {model_fit[[x]]$var.coef[z,z]})^0.5,4)
          )
          colnames(regression_coefficients_each_outcome) <- paste0(rv$entry[[1]][x], "_", colnames(regression_coefficients_each_outcome))
          return(regression_coefficients_each_outcome)
        }))
      )
      model_fit_statistics <- cbind.data.frame(
        parameter = c("AIC", "BIC", "-2 log likelihood"),
        do.call(cbind.data.frame, lapply(1:length(rv$entry[[1]]), function(x) {
          model_fit_statistics_each_outcome <- cbind.data.frame(
            statistic = c(model_fit[[x]]$aicc, model_fit[[x]]$bic, model_fit[[x]]$loglik)
          )
          colnames(model_fit_statistics_each_outcome) <- paste0(rv$entry[[1]][x], "_", colnames(model_fit_statistics_each_outcome))
          return(model_fit_statistics_each_outcome)
        }))
      )
      model_fit_statistics_display <- cbind.data.frame(
        parameter = c("AIC", "BIC", "-2 log likelihood"),
        do.call(cbind.data.frame, lapply(1:length(rv$entry[[1]]), function(x) {
          model_fit_statistics_each_outcome <- cbind.data.frame(
            statistic = round(c(model_fit[[x]]$aicc, model_fit[[x]]$bic, model_fit[[x]]$loglik),4)
          )
          colnames(model_fit_statistics_each_outcome) <- paste0(rv$entry[[1]][x], "_", colnames(model_fit_statistics_each_outcome))
          return(model_fit_statistics_each_outcome)
        }))
      )
    } else {
      if (model_fit$call[[1]] == "VAR") {
        regression_coefficients <- cbind.data.frame(
          coefficients = names(model_fit$varresult[[1]]$coefficients),
          do.call(cbind.data.frame, lapply(1:length(rv$entry[[1]]), function(x) {
            regression_coefficients_each_outcome <- cbind.data.frame(
              point_estimate = model_fit$varresult[[x]]$coefficients,
              se = sapply(1:length(model_fit$varresult[[1]]$coefficients), function(z) {vcov(model_fit$varresult[[x]])[z,z]})^0.5
            )
            colnames(regression_coefficients_each_outcome) <- paste0(rv$entry[[1]][x], "_", colnames(regression_coefficients_each_outcome))
            return(regression_coefficients_each_outcome)
          }))
        )
        regression_coefficients_display <- cbind.data.frame(
          coefficients = names(model_fit$varresult[[1]]$coefficients),
          do.call(cbind.data.frame, lapply(1:length(rv$entry[[1]]), function(x) {
            regression_coefficients_each_outcome <- cbind.data.frame(
              point_estimate = round(model_fit$varresult[[x]]$coefficients,4),
              se = round(sapply(1:length(model_fit$varresult[[1]]$coefficients), function(z) {vcov(model_fit$varresult[[x]])[z,z]})^0.5,4)
            )
            colnames(regression_coefficients_each_outcome) <- paste0(rv$entry[[1]][x], "_", colnames(regression_coefficients_each_outcome))
            return(regression_coefficients_each_outcome)
          }))
        )
      } else {
        regression_coefficients <- cbind.data.frame(
          coefficients = colnames(model_fit$deterministic),
          t(model_fit$deterministic)
        )
        regression_coefficients_display <- cbind.data.frame(
          coefficients = colnames(model_fit$deterministic),
          sapply(2:ncol(regression_coefficients), function(y) {round(regression_coefficients[,y],4)})
        )
      }
      model_fit_statistics <- cbind.data.frame(
        parameter = c("AIC", "BIC", "-2 log likelihood"),
        statistic = c(AIC(model_fit), BIC(model_fit), logLik(model_fit))
      )
      model_fit_statistics_display <- cbind.data.frame(
        parameter = c("AIC", "BIC", "-2 log likelihood"),
        statistic = round(c(AIC(model_fit), BIC(model_fit), logLik(model_fit)),4)
      )
    }
    results <- suppressWarnings(function.rbind_different_column_numbers(list = list(results, regression_coefficients, model_fit_statistics, data_forecast)))
    results_display <- suppressWarnings(function.rbind_different_column_numbers(list = list(results_display, regression_coefficients_display, model_fit_statistics_display, data_forecast_display)))
    display_table <- TRUE
    display_plot <- TRUE
    analysis_outcome <- "Successful"
  } else {
    results <- data.frame(
      `Analysis number` = paste0("AN",formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0)),
      `Analysis type` = paste0(Predefined_lists$main_menu[Predefined_lists$menu_short ==  rv$first_menu_choice], ifelse(! is.na(rv$second_menu_choice), paste0(": ", rv$second_menu_choice), "")),
      `Analysis outcome` = "Unsuccessful",
      `Reason` = time_series$date,
      check.names = FALSE
    )
    results_display <- results
    plots_list <- ""
    plots_list_display <- ""
    display_table <- TRUE
    display_plot <- FALSE
    analysis_outcome <- "Unsuccessful"
  }
  function_output <- list(analysis_outcome = analysis_outcome, plan = plan, code = code, results = results, results_display = results_display, plots_list = plots_list,   plots_list_display =   plots_list_display, selections = selections, display_table = display_table, display_plot = display_plot)
  return(function_output)
}

function.read_data <- function(data_file_path) {
  data <- read.csv(data_file_path, check.names = FALSE, na.strings = c(""," ","  ", "NA"))
  if (nrow(data) > 0) {
    func.recognise_date <- function(data) {
      lapply(1:ncol(data), function(y) {
        if (is.numeric(data[,y])) {
          field_type <- "numeric"
          converted_field <- NA
        } else if (length(data[!is.na(data[,y]), y]) == 0) {
          field_type <- "unknown"
          converted_field <- NA
        } else {
          character_field <- as.character(data[,y])
          character_field <- character_field[! is.na(character_field)]
          if (length(character_field) > 0) {
            # Number of characters must be 10 in a date field
            characters_count <- nchar(character_field)
            if (length(unique(characters_count)) > 1) {
              field_type <- "non-date character field"
              converted_field <- NA
            } else if (unique(characters_count) != 10) {
              field_type <- "non-date character field"
              converted_field <- NA
            } else {
              # If forward slash is present, it must be exactly two
              forward_slash_count <- str_count(character_field, "/")
              if (length(unique(forward_slash_count)) > 1) {
                field_type <- "non-date character field"
                converted_field <- NA
              } else if (unique(forward_slash_count) != 2) {
                # If hyphen is present, it must be exactly two
                hyphen_count <- str_count(character_field, "-")
                if (length(unique(hyphen_count)) > 1) {
                  field_type <- "non-date character field"
                  converted_field <- NA
                } else if (unique(hyphen_count) != 2) {
                  # If dot is present, it must be exactly two
                  dot_count <- str_count(character_field, "\\.")
                  if (length(unique(dot_count)) > 1) {
                    field_type <- "non-date character field"
                    converted_field <- NA
                  } else if (unique(dot_count) != 2) {
                    field_type <- "non-date character field"
                    converted_field <- NA
                  } else {
                    field_split <- do.call(rbind.data.frame, str_split(character_field,"\\."))
                    colnames(field_split) <- paste0("X",1:3)
                    # Number of characters
                    nchar1 <- nchar(field_split$X1)
                    nchar2 <- nchar(field_split$X2)
                    nchar3 <- nchar(field_split$X3)
                    unique_nchar1 <- unique(nchar1)
                    unique_nchar2 <- unique(nchar2)
                    unique_nchar3 <- unique(nchar3)
                    if ((length(unique_nchar1) > 1) | (length(unique_nchar2) > 1) | (length(unique_nchar3) > 1)) {
                      field_type <- "non-date character field"
                      converted_field <- NA
                    } else {
                      combination <- paste0(unique_nchar1, unique_nchar2, unique_nchar3)
                      if (combination == "224") {
                        # Check whether the first field or second field is month
                        converted_field <- try(as.POSIXct(paste0(field_split$X3, "-", field_split$X2, "-", field_split$X1), tz = "GMT"), silent = TRUE)
                        if (str_detect(converted_field[[1]][1], "Error")) {
                          converted_field <- try(as.POSIXct(paste0(field_split$X3, "-", field_split$X1, "-", field_split$X2), tz = "GMT"), silent = TRUE)
                        }
                        if (str_detect(converted_field[[1]][1], "Error")) {
                          field_type <- "non-date character field"
                          converted_field <- NA
                        } else {
                          field_type <- "date"
                        }
                      } else if (combination == "422") {
                        converted_field <- try(as.POSIXct(paste0(field_split$X1, "-", field_split$X2, "-", field_split$X3), tz = "GMT"), silent = TRUE)
                        if (str_detect(converted_field[[1]][1], "Error")) {
                          field_type <- "non-date character field"
                          converted_field <- NA
                        } else {
                          field_type <- "date"
                        }
                      } else {
                        field_type <- "non-date character field"
                        converted_field <- NA
                      }
                    }
                  }
                } else {
                  field_split <- do.call(rbind.data.frame, str_split(character_field,"-"))
                  colnames(field_split) <- paste0("X",1:3)
                  # Number of characters
                  nchar1 <- nchar(field_split$X1)
                  nchar2 <- nchar(field_split$X2)
                  nchar3 <- nchar(field_split$X3)
                  unique_nchar1 <- unique(nchar1)
                  unique_nchar2 <- unique(nchar2)
                  unique_nchar3 <- unique(nchar3)
                  if ((length(unique_nchar1) > 1) | (length(unique_nchar2) > 1) | (length(unique_nchar3) > 1)) {
                    field_type <- "non-date character field"
                    converted_field <- NA
                  } else {
                    combination <- paste0(unique_nchar1, unique_nchar2, unique_nchar3)
                    if (combination == "224") {
                      # Check whether the first field or second field is month
                      converted_field <- try(as.POSIXct(paste0(field_split$X3, "-", field_split$X2, "-", field_split$X1), tz = "GMT"), silent = TRUE)
                      if (str_detect(converted_field[[1]][1], "Error")) {
                        converted_field <- try(as.POSIXct(paste0(field_split$X3, "-", field_split$X1, "-", field_split$X2), tz = "GMT"), silent = TRUE)
                      }
                      if (str_detect(converted_field[[1]][1], "Error")) {
                        field_type <- "non-date character field"
                        converted_field <- NA
                      } else {
                        field_type <- "date"
                      }
                    } else if (combination == "422") {
                      converted_field <- try(as.POSIXct(paste0(field_split$X1, "-", field_split$X2, "-", field_split$X3), tz = "GMT"), silent = TRUE)
                      if (str_detect(converted_field[[1]][1], "Error")) {
                        field_type <- "non-date character field"
                        converted_field <- NA
                      } else {
                        field_type <- "date"
                      }
                    } else {
                      field_type <- "non-date character field"
                      converted_field <- NA
                    }
                  }
                }
              } else {
                field_split <- do.call(rbind.data.frame, str_split(character_field,"/"))
                colnames(field_split) <- paste0("X",1:3)
                # Number of characters
                nchar1 <- nchar(field_split$X1)
                nchar2 <- nchar(field_split$X2)
                nchar3 <- nchar(field_split$X3)
                unique_nchar1 <- unique(nchar1)
                unique_nchar2 <- unique(nchar2)
                unique_nchar3 <- unique(nchar3)
                if ((length(unique_nchar1) > 1) | (length(unique_nchar2) > 1) | (length(unique_nchar3) > 1)) {
                  field_type <- "non-date character field"
                  converted_field <- NA
                } else {
                  combination <- paste0(unique_nchar1, unique_nchar2, unique_nchar3)
                  if (combination == "224") {
                    # Check whether the first field or second field is month
                    converted_field <- try(as.POSIXct(paste0(field_split$X3, "-", field_split$X2, "-", field_split$X1), tz = "GMT"), silent = TRUE)
                    if (str_detect(converted_field[[1]][1], "Error")) {
                      converted_field <- try(as.POSIXct(paste0(field_split$X3, "-", field_split$X1, "-", field_split$X2), tz = "GMT"), silent = TRUE)
                    }
                    if (str_detect(converted_field[[1]][1], "Error")) {
                      field_type <- "non-date character field"
                      converted_field <- NA
                    } else {
                      field_type <- "date"
                    }
                  } else if (combination == "422") {
                    converted_field <- try(as.POSIXct(paste0(field_split$X1, "-", field_split$X2, "-", field_split$X3), tz = "GMT"), silent = TRUE)
                    if (str_detect(converted_field[[1]][1], "Error")) {
                      field_type <- "non-date character field"
                      converted_field <- NA
                    } else {
                      field_type <- "date"
                    }
                  } else {
                    field_type <- "non-date character field"
                    converted_field <- NA
                  }
                }
              }
            }
          } else {
            field_type <- "Unknown"
            converted_field <- NA
          }
        }
        output <- list(field_type = field_type, converted_field = converted_field)
        return(output)
      })
    }
    func.recognise_time <- function(data) {
      sapply(1:ncol(data), function(y) {
        if (is.numeric(data[,y])) {
          time_field <- FALSE
        } else if (length(data[!is.na(data[,y]), y]) == 0) {
          time_field <- FALSE
        } else {
          character_field <- as.character(data[,y])
          character_field <- character_field[! is.na(character_field)]
          if (length(character_field) > 0) {
            characters_count <- nchar(character_field)
            if (length(unique(characters_count)) > 1) {
              time_field <- FALSE
            } else if (unique(characters_count) != 8) {
              time_field <- FALSE
            } else {
              colon_count <- str_count(character_field, ":")
              if (length(unique(colon_count)) > 1) {
                time_field <- FALSE
              } else if (unique(colon_count) != 2) {
                time_field <- FALSE
              } else {
                field_split <- do.call(rbind.data.frame, str_split(character_field,":"))
                colnames(field_split) <- paste0("X",1:3)
                # Number of characters
                nchar1 <- nchar(field_split$X1)
                nchar2 <- nchar(field_split$X2)
                nchar3 <- nchar(field_split$X3)
                unique_nchar1 <- unique(nchar1)
                unique_nchar2 <- unique(nchar2)
                unique_nchar3 <- unique(nchar3)
                if ((length(unique_nchar1) > 1) | (length(unique_nchar2) > 1) | (length(unique_nchar3) > 1)) {
                  time_field <- FALSE
                } else if ((unique_nchar1 != 2) | (unique_nchar2 != 2) | (unique_nchar1 != 2)) {
                  time_field <- FALSE
                } else if ((TRUE %in% (field_split$X1 > 23)) | (TRUE %in% (field_split$X2 > 59)) | (TRUE %in% (field_split$X1 > 59))) {
                  time_field <- FALSE
                } else {
                  time_field <- TRUE
                }
              }
            }
          }
        }
      })
    }
    # Remove columns which are all NA
    not_na_length <- sapply(1:ncol(data), function(y) {length(data[!is.na(data[,y]), y])})
    data <- data[,(not_na_length != 0)]
    # Declare numerical variables - call it quantitative; declare the remaining as categorical
    any_type <- c("",colnames(data))
    quantitative <- c("",colnames(data[,unlist(lapply(data, is.numeric))]))
    categorical <- c("",colnames(data[,unlist(lapply(data, is.character))]))
    # Declare categorical variables as factors
    data[sapply(data, is.character)] <- lapply(data[sapply(data, is.character)], as.factor)
    counts <- as.vector(c("",unlist(sapply(1:length(quantitative), function(x) {
      if (quantitative[x] != "") {
        if (substr(quantitative[x],1,10) == "Number of ") {
          quantitative[x]
        }
      }
    }))))
    binary <- as.vector(c("",unlist(sapply(1:length(categorical), function(x) {
      if (categorical[x] != "") {
        if (nlevels(data[,categorical[x]]) == 2){
          categorical[x]
        }
      }
    }))))
    ordinal <- as.vector(c("",unlist(sapply(1:length(categorical), function(x) {
      if (categorical[x] != "") {
        if (!FALSE %in% (substr(levels(data[,categorical[x]]),2,2) == "_")) {
          categorical[x]
        }
      }
    }))))
    nominal <- as.vector(c("", setdiff(categorical, ordinal)))
    # Date and time variables
    processed_data <- func.recognise_date(data)
    data_converted <- lapply(1:ncol(data), function(y) {
      if (processed_data[[y]][[1]] == "date") {
        column_data <- as.character(data[,y])
        column_data[! is.na(column_data)] <- processed_data[[y]][[2]]
      } else {
        column_data <- data[,y]
      }
      return(column_data)
    })
    names(data_converted) <- colnames(data)
    data_converted <- do.call(cbind.data.frame, data_converted)
    date <- c("", colnames(data_converted)[sapply(1:ncol(data), function(y) {processed_data[[y]][[1]] == "date"})])
    time <- c("", colnames(data_converted)[func.recognise_time(data_converted)])
    output <- list(outcome = "Successful", message = "Working on uploaded data",
                   data = data, any_type = any_type,
                   quantitative = quantitative, counts = counts,
                   categorical = categorical, nominal = nominal, binary = binary, ordinal = ordinal,
                   date = date, time = time)
  } else {
    output <- list("Unsuccessful","The uploaded file did not have any data. Please upload a file with data")
  }
}

globalVariables(c("descriptive_summary"), "EQUALrepeat", add = TRUE)
function.Measurement_Error <- function(Predefined_lists, rv){
  # Lists
  plan <- {cbind.data.frame(
    analysis_number = paste0("AN", formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0)),
    first_menu_choice = rv$first_menu_choice,
    second_menu_choice = rv$second_menu_choice,
    entry_1 = paste0(rv$entry[[1]], collapse = "%_%"),
    entry_2 = paste0(rv$entry[[2]], collapse = "%_%"),
    entry_3 = paste0(rv$entry[[3]], collapse = "%_%"),
    entry_4 = paste0(rv$entry[[4]], collapse = "%_%"),
    entry_5 = paste0(rv$entry[[5]], collapse = "%_%"),
    entry_6 = paste0(rv$entry[[6]], collapse = "%_%"),
    entry_7 = paste0(rv$entry[[7]], collapse = "%_%"),
    entry_8 = paste0(rv$entry[[8]], collapse = "%_%"),
    entry_9 = paste0(rv$entry[[9]], collapse = "%_%"),
    entry_10 = paste0(rv$entry[[10]], collapse = "%_%"),
    entry_11 = paste0(rv$entry[[11]], collapse = "%_%"),
    entry_12 = paste0(rv$entry[[12]], collapse = "%_%"),
    entry_13 = paste0(rv$entry[[13]], collapse = "%_%"),
    entry_14 = paste0(rv$entry[[14]], collapse = "%_%"),
    entry_15 = paste0(rv$entry[[15]], collapse = "%_%"),
    same_row_different_row = ""
  )}
  selections <- {paste0(
    '<b>entry_1: </b>', paste0(rv$entry[[1]], collapse = "; "), '<br>',
    '<b>entry_2: </b>', paste0(rv$entry[[2]], collapse = "; "), '<br>'
  )}
  code <- {paste0(
    '# AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0), '\n',
    'rv$first_menu_choice <- "', rv$first_menu_choice, '"\n',
    'rv$second_menu_choice <- ', ifelse(is.na(rv$second_menu_choice),NA,paste0('"',rv$second_menu_choice, '"')), '\n',
    'rv$entry[[1]] <- ', ifelse(length(rv$entry[[1]]) > 1,
                                paste0('c("', paste0(rv$entry[[1]], collapse = '", "'), '")'),
                                paste0('"',rv$entry[[1]],'"')), '\n',
    'rv$entry[[2]] <- ', ifelse(length(rv$entry[[2]]) > 1,
                                paste0('c("', paste0(rv$entry[[2]], collapse = '", "'), '")'),
                                paste0('"',rv$entry[[2]],'"')), '\n',
    'AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0), '_results <- function.',rv$first_menu_choice,'(Predefined_lists, rv)', '\n',
    if(length(rv$plan) == 0){
      'if (TRUE %in% (AN0001_results$plots_list != "")) {invisible(file.rename(AN0001_results$plots_list, paste0(AN0001_results$plots_list,"_copy")))}
'
    } else {
      paste0(
        'AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$results[2,1] <- "AN',formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'"', '\n',
        'if (TRUE %in% (AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$plots_list != "")) {invisible(file.rename(AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$plots_list, str_replace_all(AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$plots_list, "/AN0001_", "/AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_")))}', '\n')
    },
    'write.table(x = AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0),'_results$results, append = TRUE, file = paste0(rv$StorageFolder, "/results.csv"), sep = ",", row.names = FALSE, col.names = FALSE, na = "", quote = FALSE, qmethod = "double")', '\n'
  )}
  # Get data
  data <- rv$import_data$data[,c(rv$entry[[1]], rv$entry[[2]])]
  data <- na.omit(data)
  # Do the analysis only if at least two rows are present
  if (nrow(data) >= 2) {
    if (rv$entry[[1]] %in% rv$import_data$categorical) {
      if (rv$entry[[1]] %in% rv$import_data$ordinal) {
        # If ordinal, Kendall correlation coefficient
        concordance_correlation_coefficient <- DescTools::KendallW(data, correct = TRUE, test = TRUE)
        correlation_coefficient <- data.frame(
          `Correlation coefficient` = concordance_correlation_coefficient$value,
          `P value` = concordance_correlation_coefficient$p.value,
          check.names = FALSE
        )
        correlation_type <- "Concordance coefficient (Kendall)"
      } else if (nlevels(data[,1]) == 2 && nlevels(data[,2]) == 2) {
        # if both rv$variables are binary, Cohen's kappa
        concordance_correlation_coefficient <- DescTools::CohenKappa(data[,1], data[,2], conf.level = 0.95)
        # p value must be calculated from the 95% confidence intervals
        se_kappa <- as.numeric((concordance_correlation_coefficient[3] - concordance_correlation_coefficient[2]) / 3.92)
        z_stat <-  as.numeric(concordance_correlation_coefficient[1] / se_kappa)
        p_value <- 2 * pnorm(abs(z_stat), lower.tail = FALSE)
        correlation_coefficient <- data.frame(
          `Correlation coefficient` = concordance_correlation_coefficient[1],
          `P value` = p_value,
          check.names = FALSE
        )
        correlation_type <- "Concordance coefficient (Cohen's kappa)"
      } else {
        # more than two levels - it is Fleiss kappa
        concordance_correlation_coefficient <- DescTools::KappaM(data, method = "Fleiss", conf.level = 0.95)
        # p value must be calculated from the 95% confidence intervals
        se_kappa <- as.numeric((concordance_correlation_coefficient[3] - concordance_correlation_coefficient[2]) / 3.92)
        z_stat <-  as.numeric(concordance_correlation_coefficient[1] / se_kappa)
        p_value <- 2 * pnorm(abs(z_stat), lower.tail = FALSE)
        correlation_coefficient <- data.frame(
          `Correlation coefficient` = concordance_correlation_coefficient[1],
          `P value` = p_value,
          check.names = FALSE
        )
        correlation_type <- "Concordance coefficient (Fleiss kappa)"
      }
      plots_list <- ""
      display_plot <- FALSE
    } else {
      concordance_correlation_coefficient <- CCC(data[,1], data[,2], ci = "z-transform", conf.level = 0.95, na.rm= TRUE)
      correlation_coefficient <- concordance_correlation_coefficient$rho.c
      colnames(correlation_coefficient) <- c("Point estimate","Lower CI", "Upper CI")
      bland_altman <- concordance_correlation_coefficient$blalt
      mean_difference <- mean(bland_altman$delta)
      se_difference <- (var(bland_altman$delta))^0.5
      correlation_type <- "concordance_correlation_coefficient"
      plot_title <- paste0(rv$entry[[1]], " versus ", rv$entry[[2]], ": Bland-Altman plot")
      plot <- ggplot(data = bland_altman,aes(x = bland_altman[,1], y = bland_altman[,2]))+ geom_point() + xlab("Means") + ylab("Differences") + xlim(round_near(min(min(data[,1]),min(data[,2]))*0.5),round_near(max(max(data[,1]),max(data[,2]))*2)) + ylim(round_near(min(bland_altman[,2])*0.5),round_near(max(bland_altman[,2])*2)) + geom_hline(yintercept=mean_difference, lty = 1, col = "gray") + geom_hline(yintercept=mean_difference - (2 * se_difference), lty = 2, col = "gray") + geom_hline(yintercept=mean_difference + (2 * se_difference), lty = 2, col = "gray") + theme(plot.title = element_text(color="navyblue", size=14, face="bold", hjust = 0.5)) + ggtitle(plot_title)
      suppressWarnings(suppressMessages(ggsave(filename = paste0(rv$StorageFolder,'/AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0), '_',
                                                                 substr(str_replace_all(plot_title, "[^[:alnum:]]","_"), 1, 80) ,'_bland_altman__plot.png'),
                                               plot = plot)))
      plots_list <- paste0(rv$StorageFolder,'/AN', formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0), '_',
                           substr(str_replace_all(plot_title, "[^[:alnum:]]","_"), 1, 80) ,'_bland_altman__plot.png')
      display_plot <- TRUE
    }
    results_display <- data.frame(
      `Analysis number` = paste0("AN",formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0)),
      `Analysis type` = paste0(Predefined_lists$main_menu[Predefined_lists$menu_short ==  rv$first_menu_choice], ifelse(! is.na(rv$second_menu_choice), paste0(": ", rv$second_menu_choice), "")),
      `Analysis outcome` = "Successful",
      `First variable` = rv$entry[[1]],
      `Second variable` = rv$entry[[2]],
      `Number of observations` = nrow(data),
      `Type of correlation coefficient` = correlation_type,
      correlation_coefficient,
      check.names = FALSE
    )
    plots_list_display <- plots_list
    analysis_outcome <- "Successful"
  } else {
    results_display <- data.frame(
      `Analysis number` = c(paste0("AN",formatC((length(rv$plan) + 1), width = 4, format = "d", flag = 0)), rep(NA, nrow(descriptive_summary)-1)),
      `Analysis type` = c(paste0(Predefined_lists$main_menu[Predefined_lists$menu_short ==  rv$first_menu_choice], ifelse(! is.na(rv$second_menu_choice), paste0(": ", rv$second_menu_choice), "")),rep(NA, nrow(descriptive_summary)-1)),
      `Analysis outcome` = c("Unsuccessful",  rep(NA, nrow(descriptive_summary)-1)),
      `First variable` = c(rv$entry[[1]], rep(NA, nrow(descriptive_summary)-1)),
      `Second variable` = c(rv$entry[[2]], rep(NA, nrow(descriptive_summary)-1)),
      `Reason for unsuccesful analysis` = "There were very few observations to perform an analysis. There must be at least two valid observations of each variable to perform a successful analysis.",
      check.names = FALSE
    )
    plots_list <- ""
    plots_list_display <- plots_list
    analysis_outcome <- "Unsuccessful"
    display_plot <- FALSE
  }
  results <- rbind.data.frame(
    colnames(results_display),
    results_display
  )
  display_table <- TRUE
  function_output <- list(analysis_outcome = analysis_outcome, plan = plan, code = code, results = results, results_display = results_display, plots_list = plots_list,   plots_list_display =   plots_list_display, selections = selections, display_table = display_table, display_plot = display_plot)
  return(function_output)
}

\name{round_near}
\alias{round_near}
\title{
Rounds a Variable to the Nearest Pretty Number
}
\description{
For some graphs, \pkg{Base} \code{pretty} function may not provide the correct rounding. This is a different algorithm suitable for the graphs produced in 'EQUAL-STATS' software.
}
\usage{
round_near(x)
}
\arguments{
\item{x}{A numeric variable.}
}
\value{
A "pretty number" suitable for use in graphs.
}
\references{
\url{https://sites.google.com/view/equal-group/home}
}
\author{
Kurinchi Gurusamy
}
\note{
This is part of a suite of functions required to allow 'EQUAL-STATS' program to run. This is unlikely to be used as a stand alone function.
}
\seealso{
\code{\link[base:pretty]{base::pretty()}}
}
\concept{ EQUAL-STATS }
\examples{
x <- 7
round_near(x)

x <- 754
round_near(x)
}

\name{function.rbind_different_column_numbers}
\alias{function.rbind_different_column_numbers}
\title{
Combine Multiple Dataframes with Different Column Numbers
}
\description{
\pkg{Base} function \code{rbind.data.frame} requires that the multiple data frames to be combined must have the same column numbers and names. For producing reports for 'EQUAL-STATS', data frames with different column numbers and names are required. This function allows this combination.
}
\usage{
function.rbind_different_column_numbers(list, include_columns)
}
\arguments{
\item{list}{A list of data frames to be combined provided as a \code{list} object.}
\item{include_columns}{Whether the column names of each data frame should be included as the first row indicated as \code{"Yes"} or \code{"No"}. Default is \code{"Yes"}. It is unsual to require \code{"No"} in this function.}
}
\value{
\item{output}{A data frame with the multiple rows combined.}
}
\references{
\url{https://sites.google.com/view/equal-group/home}
}
\author{
Kurinchi Gurusamy
}
\note{
This is part of a suite of functions required to allow 'EQUAL-STATS' program to run. This is unlikely to be used as a stand alone function.
}
\seealso{
\code{\link[base:rbind.data.frame]{base::rbind.data.frame()}}
}
\concept{ EQUAL-STATS }
\examples{
# Create a simulated data frames
df <- cbind.data.frame(
  Age = rnorm(3000, 40, 10),
  Height = rnorm(3000,165,15),
  `Length of hospital stay` = rpois(3000, 8),
  `Number of infections` = rpois(3000, 3)
)
# Calculate means and standard deviations for normally distributed variables
# and median and upper and lower quartiles for non-normally distributed variables
summary_measures <- lapply(1:ncol(df), function(y) {
  if (y<=2) {
    output <- cbind.data.frame(
      Variable = colnames(df)[y],
      Mean = mean(df[,y]),
      `Standard deviation` = sd(df[,y])
    )
  } else {
    output <- cbind.data.frame(
      Variable = colnames(df)[y],
      Median = quantile(df[,y], probs = 0.5),
      `Lower quartile` = quantile(df[,y], probs = 0.25),
      `Upper quartile` = quantile(df[,y], probs = 0.75)
    )
  }
  return(output)
})
# Combine the normally and non-normally distributed variables
normally_distributed_variables <- rbind.data.frame(summary_measures[[1]], summary_measures[[2]])
non_normally_distributed_variables <- rbind.data.frame(summary_measures[[3]], summary_measures[[4]])
# Combining the variables in a single data frame using
# rbind.data.frame causes error
combined_data_frame <- try(rbind.data.frame(normally_distributed_variables,
non_normally_distributed_variables))
combined_data_frame
# Combining the variables in a single data_frame using
# function.rbind_different_column_numbers does not cause error
# Note that data frames must be supplied as a list
# (any number of data frames can be present in the list)
# Final function ####
combined_data_frame_new_function <- function.rbind_different_column_numbers(
list(normally_distributed_variables, non_normally_distributed_variables)
)
combined_data_frame_new_function
}

\name{function.Time_Series}
\alias{function.Time_Series}
\title{
Analysis Time Series Data
}
\description{
Performs analysis of time series data and predict the variables over time. It takes into account the seasonality and trend of data and uses \pkg{forecast}, \pkg{vars}, \pkg{urca} to make the predictions. It uses \pkg{ggplot2} to create plots.
}
\usage{
function.Time_Series(Predefined_lists, rv)
}
\arguments{
\item{Predefined_lists}{A list supplied by 'EQUAL-STATS' application}
\item{rv}{A list supplied by 'EQUAL-STATS' application based on user input}
}
\value{
\item{analysis_outcome}{Whether the analysis was performed successfullly}
\item{plan}{Plan used for analysis}
\item{code}{Part of code generated for performing the analysis in a standalone version of R}
\item{results}{Analysis results}
\item{results_display}{In order to present a single table, multiple results are combined. This results in some numbers stored as text and can cause very wide tables in the shiny output. This issue is fixed wth some modifications to the results table for display purposes.}
\item{plots_list}{A list of plots generated. Returns "" if no plots are generated.}
\item{plots_list_display}{In the shiny application, only one figure is displayed. Therefore, a composite image is created from the plots for display purposes. Some analysis functions may return \code{NULL}.}
\item{selections}{Selections made by the user for display.}
\item{display_table}{Whether the results table should be displayed in the shiny app.}
\item{display_plot}{Whether the plot should be displayed in the shiny app.}
}
\references{
\url{https://sites.google.com/view/equal-group/home}
}
\author{
Kurinchi Gurusamy
}
\note{
This is part of a suite of functions required to allow 'EQUAL-STATS' program to run. This is unlikely to be used as a stand alone function.
}
\seealso{
function.submit_choices
\code{\link[stats:ts]{stats::ts()}}
\code{\link[forecast:auto.arima]{forecast::auto.arima()}}
\code{\link[forecast:forecast.fracdiff]{forecast::forecast.fracdiff()}}
\code{\link[forecast:findfrequency]{forecast::findfrequency()}}
\code{\link[forecast:ndiffs]{forecast::ndiffs()}}
\code{\link[tseries:adf.test]{tseries::adf.test()}}
\code{\link[urca:ca.jo]{urca::ca.jo()}}
\code{\link[vars:vec2var]{vars::vec2var()}}
\code{\link[vars:VAR]{vars::VAR()}}
\code{\link[viridisLite:viridis]{viridisLite::viridis()}}
\code{\link[ggplot2:ggplot]{ggplot2::ggplot()}}
\code{\link[cowplot:plot_grid]{cowplot::plot_grid()}}
}
\concept{ EQUAL-STATS }
\examples{
# Create simulated data ####
data <- cbind.data.frame(
  Date = as.Date("2023-04-01") + 0:99,
  `Currency exchange rate`= rnorm(100, mean = 95, sd = 5)
)
# Simulate lists provided by EQUAL-STATS
Predefined_lists <- list(
  main_menu = c(
    'Calculate summary measures',
    'Create plots',
    'Check distribution',
    'Compare sample mean versus population mean',
    'Compare groups/variables (independent samples)',
    'Compare groups/variables (paired samples or repeated measures)',
    'Find the correlation (quantitative variables)',
    'Calculate measurement error',
    'Find the diagnostic accuracy (primary data)',
    'Perform sample size and power calculations (primary data)',
    'Perform survival analysis',
    'Perform regression analysis',
    'Analyse time series',
    'Perform mixed-effects regression',
    'Perform multivariate regression',
    'Generate hypothesis',
    'Perform sample size and power calculations (effect size approach)',
    'Make correct conclusions (effect size approach)',
    'Find the diagnostic accuracy (tabulated data)'
  ),
  menu_short = c(
    'Summary_Measures',
    'Create_Plots',
    'Check_Distribution',
    'Compare_Sample_Pop_Means',
    'Compare_Groups',
    'Repeated_Measures',
    'Correlation',
    'Measurement_Error',
    'Diagnostic_Accuracy_Primary',
    'Sample_Size_Calculations_Primary',
    'Survival_Analysis',
    'Regression_Analysis',
    'Time_Series',
    'Mixed_Effects_Regression',
    'Multivariate_Regression',
    'Generate_Hypothesis',
    'Sample_Size_Calculations_Effect_size',
    'Make_Conclusions_Effect_size',
    'Diagnostic_Accuracy_Tables'
  )
)
entry <- list()
entry <- lapply(1:15, function(x) entry[[x]] <- '')
rv <- list(
  StorageFolder = tempdir(),
  first_menu_choice = NA,
  second_menu_choice = NA,
  entry = entry,
  import_data = NULL,
  same_row_different_row = NA,
  submit_button_to_appear = FALSE,
  summary_measures_choices = c("EQUAL-STATS choice", "Total observations",
  "Missing observations", "Available observations"),
  analysis_outcome = list(),
  code = list(),
  plan = list(),
  results = list(),
  plots_list = list(),
  reports = list()
)
# Store the data in a folder
data_file_path = paste0(tempdir(), "/data.csv")
write.csv(data, file = data_file_path, row.names = FALSE, na = "")
# Load the necessary packages and functions ####
library(stringr)
library(forecast)
library(viridisLite)
library(cowplot)
library(ggplot2)
rv$import_data <- function.read_data(data_file_path)
rv$first_menu_choice <- "Time_Series"
rv$second_menu_choice <- NA
rv$entry[[1]] <- "Currency exchange rate"
rv$entry[[2]] <- "Date"
rv$entry[[3]] <- ""
rv$entry[[4]] <- ""
rv$entry[[5]] <- "Univariate"
rv$entry[[6]] <- 10
# Final function ####
Results <- function.Time_Series(Predefined_lists, rv)
}

\name{function.read_data}
\alias{function.read_data}
\title{
Read a CSV File and Classify Variable Type
}
\description{
When an user uploads a file in 'EQUAL-STATS' program, the program can automatically classify the variable types based on the nature of the data uploaded. The data and data types are stored in memory. This then determines the options available for questions and the analysis performed. The variable types can be altered using function.read_metadata.
}
\usage{
function.read_data(data_file_path)
}
\arguments{
\item{data_file_path}{The path to the data file.}
}
\value{
\item{outcome}{Whether the import was successful.}
\item{message}{The message displayed to the user after the processing. This message also contains the reason for failure if the import was unsuccessful.}
\item{data}{Imported data}
\item{any_type}{All variables in the data}
\item{quantitative}{Quantitative variables in the data}
\item{counts}{Count variables in the data}
\item{categorical}{Categorical variables in the data}
\item{nominal}{Categorical variables without any order in the data}
\item{binary}{Categorical variables with only two possible categories (factors/levels) in the data}
\item{ordinal}{Ordered categorical variables}
\item{date}{Any variables that appear like date}
\item{time}{Any variables that appear like time}
}
\references{
\url{https://sites.google.com/view/equal-group/home}
}
\author{
Kurinchi Gurusamy
}
\note{
This is part of a suite of functions required to allow 'EQUAL-STATS' program to run. This is unlikely to be used as a stand alone function.
}
\seealso{
function.read_metadata
}
\concept{ EQUAL-STATS }
\examples{
# Create simulated data ####
data <- cbind.data.frame(
  `Subject ID` = c(
    "S0001", "S0002", "S0003", "S0004", "S0005",
    "S0006", "S0007", "S0008", "S0009", "S0010",
    "S0011", "S0012", "S0013", "S0014", "S0015",
    "S0016", "S0017", "S0018", "S0019", "S0020",
    "S0021", "S0022", "S0023", "S0024", "S0025",
    "S0026", "S0027", "S0028", "S0029", "S0030"),
  `Centre` = c(
    "C_0001", "C_0002", "C_0002", "C_0002", "C_0002",
    "C_0001", "C_0001", "C_0003", "C_0001", "C_0003",
    "C_0001", "C_0002", "C_0002", "C_0001", "C_0003",
    "C_0002", "C_0002", "C_0003", "C_0001", "C_0002",
    "C_0002", "C_0002", "C_0002", "C_0003", "C_0002",
    "C_0001", "C_0003", "C_0001", "C_0001", "C_0001"),
  `Treatment` = c(
    "Intensive rehabilitation", "Intensive rehabilitation", "Standard rehabilitation",
    "Intensive rehabilitation", "Intensive rehabilitation", "Intensive rehabilitation",
    "Intensive rehabilitation", "Intensive rehabilitation", "Intensive rehabilitation",
    "Standard rehabilitation", "Intensive rehabilitation", "Standard rehabilitation",
    "Standard rehabilitation", "Intensive rehabilitation", "Intensive rehabilitation",
    "Intensive rehabilitation", "Standard rehabilitation", "Standard rehabilitation",
    "Intensive rehabilitation", "Standard rehabilitation", "Intensive rehabilitation",
    "Intensive rehabilitation", "Standard rehabilitation", "Intensive rehabilitation",
    "Intensive rehabilitation", "Standard rehabilitation", "Standard rehabilitation",
    "Intensive rehabilitation", "Standard rehabilitation", "Intensive rehabilitation"),
  `Obesity status` = c(
    "Obese", "Non-obese", "Obese", "Non-obese", "Non-obese",
    "Obese", "Obese", "Obese", "Non-obese", "Obese",
    "Non-obese", "Non-obese", "Obese", "Non-obese", "Obese",
    "Obese", "Non-obese", "Obese", "Obese", "Obese",
    "Non-obese", "Non-obese", "Non-obese", "Obese", "Obese",
    "Non-obese", "Obese", "Obese", "Obese", "Obese"),
  `Unable to walk independently at 6 weeks` = c(
    "unable", "able", "able", "unable", "able",
    "able", "unable", "unable", "unable", "unable",
    "able", "unable", "able", "unable", "unable",
    "able", "unable", "unable", "unable", "unable",
    "able", "able", "able", "able", "unable",
    "able", "able", "unable", "able", "unable"),
  `Mobility score at 6 months` = c(
    86, 65.1, 48, 99.8, 73.4, 70, 74.7, 36.5, 64.6, 85.4,
    41.7, 60.1, 73.3, 42.4, 55.3, 47.3, 85.9, 63, 64.6, 101.8,
    108.1, 72.3, 96.4, 87.5, 66.2, 92.9, 47.7, 55.8, 56.4, 133.8),
  `Pain at 6 weeks` = c(
    "3_severe", "1_mild", "1_mild", "2_moderate", "1_mild",
    "1_mild", "2_moderate", "2_moderate", "1_mild", "3_severe",
    "1_mild", "2_moderate", "1_mild", "3_severe", "3_severe",
    "1_mild", "2_moderate", "3_severe", "2_moderate", "2_moderate",
    "1_mild", "1_mild", "1_mild", "1_mild", "2_moderate",
    "1_mild", "1_mild", "2_moderate", "1_mild", "2_moderate"),
  `Number of falls within 6 months` = c(
    3, 2, 3, 2, 2, 1, 4, 2, 2, 5,
    3, 2, 2, 2, 5, 3, 2, 2, 3, 4,
    3, 1, 2, 2, 2, 7, 2, 1, 1, 8),
  `Mobility score at 12 months` = c(
    90, 69.1, 52, 103.8, 77.4, 74, 78.7, 40.5, 68.6, 89.4,
    45.7, 64.1, 77.3, 46.4, 59.3, 51.3, 89.9, 67, 68.6, 105.8,
    112.1, 76.3, 100.4, 91.5, 70.2, 96.9, 51.7, 59.8, 60.4, 137.8)
)
# Store this in a folder
data_file_path = paste0(tempdir(), "/data.csv")
write.csv(data, file = data_file_path, row.names = FALSE, na = "")
# Load the necessary packages
library(stringr)
# Final function ####
imported_data_types <- function.read_data(data_file_path)
}
